/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

import  edu.columbia.ais.portal.utils.*;
import  org.jasig.portal.services.*;
import  java.text.SimpleDateFormat;
import  java.util.Date;
import  java.util.HashMap;
import  org.xml.sax.InputSource;
import  java.io.*;
import  java.util.Calendar;
import  java.util.Arrays;
import  org.jasig.portal.security.IPerson;
//import org.w3c.dom.Document;
import  org.w3c.dom.Node;
import  org.w3c.dom.NodeList;
import  org.w3c.dom.Element;
import  org.w3c.dom.Text;
import  org.w3c.dom.Document;
import javax.xml.parsers.*;
import  java.sql.Connection;
import  java.sql.Statement;
import  java.sql.ResultSet;
import  java.sql.Timestamp;
import  java.io.StringReader;
import  java.io.StringWriter;
import  java.util.Hashtable;
import  java.util.Enumeration;
import  java.util.Iterator;
import  java.util.Map.Entry;


/**
 *
 * Announcement is the Java class representing a single announcement
 * along with its XML representation.
 *
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.1 $
 */
public class Announcement {
    String ID;
    String from;
    String content;
    int priority;
    String link;
    Timestamp created;
    Timestamp appears;
    Timestamp expires;
    String topicID;
    boolean draft;
    boolean locked;
    String locker;
    Date locktime;
    Timestamp lastModDate;
    protected static String[] priorities = new String[6];
    protected static Document blank;
    String title;
    String summary;


    static {
        priorities[1] = "lowest";
        priorities[2] = "low";
        priorities[3] = "normal";
        priorities[4] = "high";
        priorities[5] = "highest";
        blank = org.jasig.portal.utils.DocumentFactory.getNewDocument();
    }

    /**
     * Create a new Announcement for a topic by ID
     * @param     String tID
     */
    public Announcement (String topicID) {
        this.topicID = topicID;
        this.draft = false;
    }

    /**
     * Create a new Announcement for a topic by ID and gather all
     * the announcement data from an "announcement" XML element
     *
     * @param     String topicID
     * @param     Element xmsg
     */
    public Announcement (String topicID, Element xmsg) {
        try {
            this.topicID = topicID;
            this.draft = false;
            this.from = xmsg.getAttribute("from");
            this.ID = xmsg.getAttribute("id");
            SimpleDateFormat df = new SimpleDateFormat();
            if (xmsg.getAttribute("expires") != "") {
                this.expires = convertToTimeStamp(xmsg.getAttribute("expires"));
            }
            if (xmsg.getAttribute("generated.on") != "") {
                this.created = convertToTimeStamp(xmsg.getAttribute("generated.on"));
            }
            if (xmsg.getAttribute("priority") != "") {
                this.setPriority(Arrays.asList(priorities).indexOf(xmsg.getAttribute("priority")));
            }
            NodeList props = xmsg.getElementsByTagName("property");
            for (int x = 0; x < props.getLength(); x++) {
                Element prop = (Element)props.item(x);
                if (prop.getAttribute("name").equals("appears")) {
                    this.appears = convertToTimeStamp(prop.getAttribute("value"));
                }
                else if (prop.getAttribute("name").equals("draft")) {
                    if (prop.getAttribute("value").equals("true")) {
                        this.draft = true;
                    }
                }
            }


            Node document = (xmsg.getElementsByTagName("document")).item(0);
            Node msg_title = ( ((Element) document).getElementsByTagName("title")).item(0);
            this.title = ((Element) msg_title).getFirstChild().getNodeValue();
            Node msg_summary = ( ((Element) document).getElementsByTagName("summary")).item(0);
            this.summary = ((Element) msg_summary).getFirstChild().getNodeValue();

            NodeList docs = xmsg.getElementsByTagName("document");
            for (int x = 0; x < docs.getLength(); x++) {
                NodeList datas = ((Element)docs.item(x)).getElementsByTagName("data");
                if (datas.getLength() > 0) {
                    if (datas.item(0).getFirstChild() != null) {
                        this.content = ((Element)datas.item(0)).getFirstChild().getNodeValue();
                    }
                }
                //Element child = (Element) ((Node) docs.item(x)).getFirstChild();
                NodeList datarefs = ((Element)docs.item(x)).getElementsByTagName("data.reference");
                if (datarefs.getLength() > 0) {
                    this.link = ((Element)datarefs.item(0)).getAttribute("href");
                }
            }
            //LogService.instance().log(LogService.DEBUG,"Added msg #"+this.ID.toString()+" to topic "+this.topicID.toString());
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }


    /**
     * put your documentation comment here
     * @param time
     * @return
     */
    protected static Timestamp convertToTimeStamp (String time) {
        Timestamp rs = null;
        try {
            rs = new Timestamp(0).valueOf(time);
        } catch (java.lang.IllegalArgumentException e) {
            // OK, probably came in in numeric form
            try {
                String newTime = time.substring(0, 4) + "-" + time.substring(4,
                        6) + "-" + time.substring(6, 8) + " ";
                newTime = newTime + time.substring(8, 10) + ":" + time.substring(10,
                        12) + ":" + time.substring(12);
                rs = new Timestamp(0).valueOf(newTime);
            } catch (Exception e2) {
                LogService.log(LogService.ERROR, "Announcemnt:convertToTimestamp(): unable to convert String "
                        + time + " to valid timestamp " + e2);
            }
        }
        return  rs;
    }

    /**
     * put your documentation comment here
     * @param ts
     * @return
     */
    protected String Timestamp2NumberString (Timestamp ts) {
        String rs = "";
        char c1 = '-';
        char c2 = ':';
        char c3 = ' ';
        if (ts != null) {
            char[] exchars = ts.toString().toCharArray();
            StringBuffer exb = new StringBuffer();
            for (int e = 0; e < exchars.length; e++) {
                if ((exchars[e] != c1) && (exchars[e] != c2) && (exchars[e] != c3)) {
                    exb.append(exchars[e]);
                }
            }
            rs = exb.toString();
        }
        return  rs;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Document retrieveXML () {
      Document rDoc = (Document) blank.cloneNode(false);
      rDoc.appendChild(retrieveXML(rDoc));
      return  rDoc;
    }

    public Element retrieveXML(Document rDoc){
      Element m = (Element)rDoc.createElement("message");
      m.setAttribute("from", this.from);
      m.setAttribute("generated.on", Timestamp2NumberString(this.created));
      m.setAttribute("id", this.ID);
      m.setAttribute("priority", priorities[this.getPriority()]);
      if (this.expires != null) {
          m.setAttribute("expires", Timestamp2NumberString(expires));
      }
      Element tp = (Element)rDoc.createElement("property");
      tp.setAttribute("name", "topicID");
      tp.setAttribute("value", this.topicID);
      m.appendChild(tp);
      if (this.appears != null) {
          Element app = (Element)rDoc.createElement("property");
          app.setAttribute("name", "appears");
          app.setAttribute("value", Timestamp2NumberString(appears));
          m.appendChild(app);
      }
      if (this.draft) {
          Element dra = (Element)rDoc.createElement("property");
          dra.setAttribute("name", "draft");
          dra.setAttribute("value", "true");
          m.appendChild(dra);
      }
      Element mdocu = (Element)rDoc.createElement("document");
      mdocu.setAttribute("uri", "message[" + this.ID + "]/document[1]");
      
      Element msg_title = (Element) rDoc.createElement("title");
      msg_title.setAttribute("content.type", "text/plain");
      msg_title.setAttribute("encoding", "UTF-8");
      msg_title.appendChild(rDoc.createTextNode(this.title));
      mdocu.appendChild(msg_title);
      Element msg_summary = (Element) rDoc.createElement("summary");
      msg_summary.setAttribute("msg_id", this.ID);
      msg_summary.setAttribute("topic_id", this.topicID);
      msg_summary.setAttribute("content.type", "text/html");
      msg_summary.setAttribute("encoding", "UTF-8");
      msg_summary.appendChild(rDoc.createTextNode(this.summary));
      mdocu.appendChild(msg_summary);

      Element data = (Element)rDoc.createElement("data");
      data.setAttribute("content.type", "text/plain");
      data.setAttribute("encoding", "UTF-8");
      data.appendChild(rDoc.createTextNode(this.content));
      mdocu.appendChild(data);
      m.appendChild(mdocu);
      //get any links associated with message
      if (link != null) {
          Element dr = (Element)rDoc.createElement("data.reference");
          dr.setAttribute("content.type", "unknown");
          dr.setAttribute("href", link);
          Element ddoc = (Element)rDoc.createElement("document");
          ddoc.setAttribute("uri", "message[" + ID + "]/document[1]");
          ddoc.appendChild(dr);
          m.appendChild(ddoc);
      }
      Element msg = m;
      if (expires != null) {
          Element eMM = rDoc.createElement("property");
          eMM.setAttribute("name", "eMM");
          eMM.setAttribute("value", new Integer(expires.getMonth() + 1).toString());
          msg.insertBefore(eMM, msg.getFirstChild());
          Element eDD = rDoc.createElement("property");
          eDD.setAttribute("name", "eDD");
          eDD.setAttribute("value", new Integer(expires.getDate()).toString());
          msg.insertBefore(eDD, msg.getFirstChild());
          Element eYYYY = rDoc.createElement("property");
          eYYYY.setAttribute("name", "eYYYY");
          eYYYY.setAttribute("value", new Integer(expires.getYear() +
                  1900).toString());
          msg.insertBefore(eYYYY, msg.getFirstChild());
      }
      if (appears != null) {
          Element aMM = rDoc.createElement("property");
          aMM.setAttribute("name", "aMM");
          aMM.setAttribute("value", new Integer(appears.getMonth() + 1).toString());
          msg.insertBefore(aMM, msg.getFirstChild());
          Element aDD = rDoc.createElement("property");
          aDD.setAttribute("name", "aDD");
          aDD.setAttribute("value", new Integer(appears.getDate()).toString());
          msg.insertBefore(aDD, msg.getFirstChild());
          Element aYYYY = rDoc.createElement("property");
          aYYYY.setAttribute("name", "aYYYY");
          aYYYY.setAttribute("value", new Integer(appears.getYear() +
                  1900).toString());
          msg.insertBefore(aYYYY, msg.getFirstChild());
      }
      //String priority = msg.getAttribute("priority");
      if (priority > 0) {
          Element pnum = rDoc.createElement("property");
          pnum.setAttribute("name", "prioritynumber");
          pnum.setAttribute("value", String.valueOf(priority));
          msg.insertBefore(pnum, msg.getFirstChild());
      }
      return m;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getLastModDate () {
        return  lastModDate;
    }

    /**
     * put your documentation comment here
     * @param ts
     */
    public void setLastModDate (Timestamp ts) {
        this.lastModDate = ts;
    }

    /**
     * put your documentation comment here
     */
    protected void update () {
        this.setLastModDate(new Timestamp(Calendar.getInstance().getTime().getTime()));
    }

    /**
     * put your documentation comment here
     * @param newID
     */
    public void setID (String newID) {
        if (lastModDate == null) {
            ID = newID;
        }
        else if (!newID.equals(ID)) {
            ID = newID;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getID () {
        return  ID;
    }

    /**
     * put your documentation comment here
     * @param val
     */
    public void setDraft (boolean val) {
        if (lastModDate == null) {
            draft = val;
        }
        else if (draft != val) {
            draft = val;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public boolean getDraft () {
        return  draft;
    }

    /**
     * put your documentation comment here
     * @param s
     */
    public void setContent (String s) {
        if (lastModDate == null) {
            content = s;
        }
        else if (!s.equals(content)) {
            content = s;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getContent () {
        return  content;
    }

    /**
     * put your documentation comment here
     * @param f
     */
    public void setFrom (String f) {
        if (lastModDate == null) {
            from = f;
        }
        else if (!f.equals(from)) {
            from = f;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getFrom () {
        return  from;
    }

    /**
     * put your documentation comment here
     * @param URI
     */
    public void setLink (String URI) {
        if (lastModDate == null) {
            link = URI;
        }
        else if (!URI.equals(link)) {
            link = URI;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getLink () {
        return  link;
    }

    /**
     * put your documentation comment here
     * @param ap
     */
    public void setAppears (Timestamp ap) {
        if (lastModDate == null) {
            appears = ap;
        }
        else if (!ap.equals(appears)) {
            appears = ap;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getAppears () {
        return  appears;
    }

    /**
     * put your documentation comment here
     * @param ex
     */
    public void setExpires (Timestamp ex) {
        if (lastModDate == null) {
            expires = ex;
        }
        else if (!ex.equals(expires)) {
            this.expires = ex;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getExpires () {
        return  expires;
    }

    /**
     * put your documentation comment here
     * @param tc
     */
    public void setCreated (Timestamp tc) {
        if (lastModDate == null) {
            created = tc;
        }
        else if (!tc.equals(created)) {
            created = tc;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getCreated () {
        return  created;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getTopicID () {
        return  topicID;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public int getPriority () {
        return  priority;
    }

    /**
     * put your documentation comment here
     * @param p
     */
    public void setPriority (int p) {
        if (lastModDate == null) {
            priority = p;
        }
        else if ((p != priority) && (6 > p) && (p > 0)) {
            priority = p;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @param p
     */
    public void setPriority (String p) {
        int pr = Arrays.asList(priorities).indexOf(p);
        setPriority(pr);
    }

    //added
    public void setTitle(String s) {
        if (lastModDate == null) {
            title = s;
          } else if (!s.equals(title)) {
            title = s;
            update();
        }
    }

    /**
     * put your documentation comment here
     *
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * put your documentation comment here
     *
     * @param s
     */
    public void setSummary(String s) {

        if (lastModDate == null) {
            summary = s;
        } else if (!s.equals(summary)) {
            summary = s;
            update();
        }
    }

    /**
     * put your documentation comment here
     *
     * @return
     */
    public String getSummary() {
        return summary;
    }

}

