/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */


package  edu.columbia.ais.portal.channels.announcements;

import  edu.columbia.ais.portal.utils.*;
import  org.jasig.portal.channels.permissionsmanager.*;
import  org.jasig.portal.channels.groupsmanager.*;
import  java.io.*;
import  java.util.Properties;
import  org.jasig.portal.IMultithreadedCacheable;
import  org.jasig.portal.ICacheable;
import  org.jasig.portal.IPermissible;
import  org.jasig.portal.PortalEvent;
import  org.jasig.portal.IServant;
import  org.jasig.portal.IChannel;
import  org.jasig.portal.IMultithreadedChannel;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.ChannelRuntimeProperties;
import  org.jasig.portal.StylesheetSet;
import  org.jasig.portal.ChannelStaticData;
import  org.jasig.portal.groups.*;
import  org.jasig.portal.security.provider.*;
import  org.jasig.portal.security.*;
import  org.jasig.portal.security.IPerson;
import  org.jasig.portal.utils.*;
import  org.jasig.portal.services.*;
import  java.sql.Timestamp;
import  java.util.Calendar;
import  java.util.HashMap;
import  java.util.ArrayList;
//import org.w3c.dom.Document;
import  org.w3c.dom.Node;
import  org.w3c.dom.NodeList;
import  org.w3c.dom.Element;
import  org.w3c.dom.Text;
import  org.w3c.dom.Document;
import javax.xml.parsers.*;
import  java.sql.Connection;
import  java.sql.Statement;
import  java.sql.ResultSet;
import  java.io.StringReader;
import  java.io.StringWriter;
import  java.util.Hashtable;
import  java.util.Vector;
import  java.util.Enumeration;
import  java.util.Iterator;
import  java.util.Map.Entry;
import  org.jasig.portal.ChannelCacheKey;
import org.xml.sax.ContentHandler;


/** The subscribable Announcements IChannel class for uPortal 2.x
 * @author Alex Vigdor
 * @version $Revision: 1.1 $
 *
 *  modified by S. QIANG, qiang@uhp-nancy.fr (Esup-Portail) 2004/03/28
 * Last modified by S. QIANG 2004/11/05
 */
public class CAnnouncements
        implements IMultithreadedChannel, Constants, IPermissible, IMultithreadedCacheable {

private static final String XML_DOC_KEY = CAnnouncements.class.getName()+"$XML_DOC_KEY";

    private static HashMap activities = null;
    protected static HashMap sessions = new HashMap();
    // Initialize StylesheetSet
    private static final String sslLocation = "CAnnouncements.ssl";

    /**
     * put your documentation comment here
     */
    public CAnnouncements () {
        init();
    }

    /**
     * put your documentation comment here
     * @param uid
     * @return
     */
    protected synchronized CAnnouncementsSessionData getSessionData (String uid) {
        if (sessions.get(uid) == null) {
            CAnnouncementsSessionData csd = new CAnnouncementsSessionData();
            csd.uid = uid;
            sessions.put(uid, csd);
        }
        return  (CAnnouncementsSessionData)sessions.get(uid);
    }

    /**
     * put your documentation comment here
     */
    protected synchronized void init () {
        if (activities == null) {
            activities = new HashMap();
            try {
                activities.put("Subscribe", subscribeTopicBtnTxt);
                activities.put("DraftAnnouncement", draftAnnouncementsBtnTxt);
                activities.put("PostAnnouncement", postAnnouncementsBtnTxt);
                activities.put("EditAnnouncement", editAnnouncementsBtnTxt);
                activities.put("DeleteAnnouncement", deleteAnnouncementsBtnTxt);
                //activities.put("CreateTopic","Create Topics");
                activities.put("EditTopic", editTopicBtnTxt);
                activities.put("DeleteTopic", deleteTopicBtnTxt);
                activities.put("AssignPermissions", assignPermissionsForTopicBtnTxt);
            } catch (Exception e) {
                LogService.log(LogService.ERROR, e);
            }
        }
    }

    /**
     * put your documentation comment here
     * @param uid
     * @return
     */
    public ChannelRuntimeProperties getRuntimeProperties (String uid) {
        return  new ChannelRuntimeProperties();
    }

    /**
     * put your documentation comment here
     * @param ev
     * @param uid
     */
    public void receiveEvent (PortalEvent ev, String uid) {
        if (ev.getEventNumber() == ev.EDIT_BUTTON_EVENT) {
        // Switch to edit mode
        //m_currentState = EDITMODE;
        // Clear the content cache
        //m_cachedContent = null;
        }
        if (ev.getEventNumber() == ev.SESSION_DONE) {
            sessions.remove(uid);
        }
    }

    // Receive ChannelStaticData from the portal
    public void setStaticData (ChannelStaticData sD, String uid) {
        CAnnouncementsSessionData session = getSessionData(uid);
        session.staticData = sD;
        session.instanceIdentifier=sD.getParameter("instance");

        //System.out.println("Set announcements instance to "+session.instanceIdentifier);
        try {
            IEntityGroup admin = GroupService.getDistinguishedGroup(GroupService.PORTAL_ADMINISTRATORS);
            IGroupMember me = AuthorizationService.instance().getGroupMember(sD.getAuthorizationPrincipal());
            session.member = me;
            if (admin.deepContains(me)) {
                session.isAdminUser = true;
            }

            try {
                            String gname = sD.getParameter("TopicAdminGroup");
                            if (gname.trim().startsWith("local.") || gname.trim().startsWith("ldap.") ) {
                                    IEntityGroup topicAdmin = GroupService.findGroup(gname);
                                    if((topicAdmin != null) && (topicAdmin.deepContains(me)))
                                      {
                                            session.isTopicAdminUser = true;
                                            LogService.log(LogService.DEBUG, "CAnouncements::setStaticData(): isTopicAdminUser=true");
                                      }
                                                            }
                            else LogService.log(LogService.ERROR, "CAnouncements::setStaticData(): TopicAdminGroup not well defined");
                    } catch (GroupsException e1) {
                            LogService.log(LogService.ERROR, "CAnouncements::setStaticData(): Problem TopicAdminGroup : "+ e1);
                    }

        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }


    }

    // Receive ChannelRuntimeData from the portal and process actions passed in it
    public void setRuntimeData (ChannelRuntimeData rD, String uid) {
        CAnnouncementsSessionData session = getSessionData(uid);
        session.startRD = Calendar.getInstance().getTime().getTime();
        LogService.log(LogService.DEBUG, this.getClass().getName() + ".setRuntimeData()");
        session.runtimeData = rD;
        //Iterator i = session.runtimeData.entrySet().iterator();
        Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
        // create servant
        if (hasValue(rD.getParameter("view"), "AssignPermissions") && session.servantChannel
                == null) {
            session.myvalidity = now;
            try {
                String[] tgts = new String[1];
                tgts[0] = (rD.getParameter("topicID"));
                session.servantChannel = CPermissionsManagerServantFactory.getPermissionsServant(this,
                session.staticData, null, null, tgts);
            } catch (Exception e) {
                LogService.log(LogService.ERROR, "CAnnouncements.setRuntimeDat(): Problem instantiating servant");
                LogService.log(LogService.ERROR,e);
                rD.setParameter("view", defaultView);
            }
        }
        if (hasValue(rD.getParameter("view"), "AssignGroups") && session.servantChannel
                == null) {
            session.myvalidity = now;
            if (hasValue(rD.getParameter("subscriberGroup"))) {
                try {
                    session.servantChannel = CGroupsManagerServantFactory.getGroupsServantforAddRemove(session.staticData,
                            rD.getParameter("subscriberGroup"));
                } catch (Exception e) {
                    LogService.log(LogService.ERROR, "CAnnouncements.setRuntimeDat(): Problem instantiating servant");
                    LogService.log(LogService.ERROR,e);
                    rD.setParameter("view", defaultView);
                }
            }
        }
        if (session.servantChannel != null) {   
          if (rD.isRenderingAsRoot()){
            try {
                LogService.log(LogService.DEBUG, "CAnouncements.setRuntimeData(): Setting Servant runtimedata");
                ((IChannel)session.servantChannel).setRuntimeData(rD);
                if (session.servantChannel.isFinished()) {
                    session.servantChannel = null;
                    //staticData.remove("view");
                    LogService.log(LogService.DEBUG, "CAnouncements.setRuntimeData(): removed servant");
                }
                else {
                    LogService.log(LogService.DEBUG, "CAnouncements.setRuntimeData(): servant Not Finished");
                }
            } catch (Exception e) {
                LogService.log(LogService.ERROR, "CAnouncements.setRuntimeDat(): Problem setting servant runtimedata "
                        + e);
            }
          }
          else{
            session.servantChannel = null;
            rD.setParameter("view", defaultView);
          }
          session.myvalidity = now;
        }
        // finished w/servant code
        if (!hasValue(rD.getParameter("command"))) {
            rD.remove("command");
        }
        else {
            session.myvalidity = now;
        }
        if (!hasValue(rD.getParameter("topicID"))) {
            if (hasValue(session.staticData.get("topicID")) && !hasValue(rD.get("view"))
                    && !hasValue(rD.get("command"))) {
                rD.setParameter("topicID", session.staticData.getParameter("topicID"));
            }
            else {
                rD.remove("topicID");
                session.staticData.remove("topicID");
            }
        }
        if (!hasValue(rD.getParameter("messageID"))) {
            if (hasValue(session.staticData.get("messageID")) && !hasValue(rD.get("view"))
                    && !hasValue(rD.get("command"))) {
                rD.setParameter("messageID", session.staticData.getParameter("messageID"));
            }
            else {
                rD.remove("messageID");
                session.staticData.remove("messageID");
            }
        }
        if (!hasValue(rD.getParameter("topicStore"))) {
            rD.remove("topicStore");
        }
        if (!hasValue(rD.getParameter("topicStoreAuthority"))) {
            rD.remove("topicStoreAuthority");
        }
        if (!hasValue(rD.getParameter("view"))) {
            if (hasValue(session.staticData.get("view"))) {
                if (hasValue(session.staticData.get("view"), "AssignGroups") ||
                        hasValue(session.staticData.get("view"), "AssignPermissions")) {
                    rD.setParameter("view", (String)session.fromViews.get(session.staticData.getParameter("view")));
                }
                else {
                  if(rD.isRenderingAsRoot()){
                    rD.setParameter("view", session.staticData.getParameter("view"));
                  }
                  else{
                    rD.setParameter("view", defaultView);
                    if (!rD.getParameter("view").equals(session.staticData.get("view"))) {
                        session.myvalidity = now;
                    }
                  }
                }
            }
            else {
                rD.setParameter("view", defaultView);
            }
        }
        else if (!rD.getParameter("view").equals(session.staticData.get("view"))) {
            session.myvalidity = now;
        }
             else { // Case where the view is the same as the previous one

                        // Check if there was a page number
                        String oldpageNum = (String) session.staticData.get("pageNumber");
                        String newPageNum = rD.getParameter("pageNumber");

                        if (newPageNum != null && !newPageNum.equals(oldpageNum)) {
                                session.myvalidity = now;
                        }

                        // Check if the topic ID is different
                        String oldTopicID = (String) session.staticData.get("topicID");
            String newTopicID = rD.getParameter("topicID");
             if (newTopicID != null && !newTopicID.equals(oldTopicID)) {
                                session.myvalidity = now;
                        }
                }

                // Keep track of last page number
                if (hasValue(rD.getParameter("pageNumber"))) {
            session.staticData.put("pageNumber", rD.getParameter("pageNumber"));
        }

        if (hasValue(rD.getParameter("fromView"))) {
            session.fromViews.put(rD.getParameter("view"), rD.getParameter("fromView"));
            //staticData.setParameter("fromView", runtimeData.getParameter("fromView"));
        }
        if (hasValue(rD.getParameter("view"))) {
            session.staticData.setParameter("view", rD.getParameter("view"));
        }
        if (hasValue(rD.getParameter("topicID"))) {
            session.staticData.setParameter("topicID", rD.getParameter("topicID"));
        }
        if (hasValue(rD.getParameter("messageID"))) {
            session.staticData.setParameter("messageID", rD.getParameter("messageID"));
        }
        if (rD.getParameter("command") != null) {
            LogService.log(LogService.DEBUG, "Getting Command");

            CommandFactory cf = CommandFactory.instance();
            IAnnouncementCommand c = cf.get(rD.getParameter("command"));
            if (c != null) {
                LogService.instance().log(LogService.DEBUG, "Executing Command "
                        + c.getClass().getName());
                c.execute(session);
            }
        }
        if (session.servantChannel == null) {
           // ViewXML.get(session);
            //Timestamp doctime = ViewXML.getCacheTimeStamp(uid);
            rD.put(XML_DOC_KEY,ViewXML.get(session));
            Timestamp maxModViewXML = session.getMaxModViewXML();
              if (maxModViewXML != null) {
               if (session.myvalidity == null || maxModViewXML.after(session.myvalidity)) {
                             session.myvalidity = maxModViewXML;
                         }
            }
        }        /*
         Enumeration names = runtimeData.getParameterNames();
         while (names.hasMoreElements()) {
         String n = (String)names.nextElement();
         LogService.instance().log(LogService.DEBUG, this.getClass().getName()
         + ".setRuntimeData() RuntimeData paramaters: " + n +
         " = " + runtimeData.getParameter(n));
         }
         Enumeration names2 = staticData.keys();
         while (names2.hasMoreElements()) {
         String n2 = (String)names2.nextElement();
         LogService.instance().log(LogService.DEBUG, this.getClass().getName()
         + ".setRuntimeData() StaticData paramaters: " + n2 +
         " = " + staticData.get(n2).toString());
         }
         */

    }

    /**
     * put your documentation comment here
     * @param o
     * @return
     */
    protected boolean hasValue (Object o) {
        boolean rval = false;
        if (o != null && !o.toString().trim().equals("")) {
            rval = true;
        }
        return  rval;
    }

    /**
     * put your documentation comment here
     * @param o
     * @param test
     * @return
     */
    protected boolean hasValue (Object o, String test) {
        boolean rval = false;
        if (hasValue(o)) {
            if (String.valueOf(o).equals(test)) {
                rval = true;
            }
        }
        return  rval;
    }

    /**
     *  Render the user's layout based on the current state of the channel
     */
    public void renderXML (ContentHandler out, String uid) {
        long time1 = Calendar.getInstance().getTime().getTime();

            CAnnouncementsSessionData session = getSessionData(uid);
            try {
            // start servant code
            if (session.servantChannel != null) {
                ((IChannel)session.servantChannel).renderXML(out);
                LogService.log(LogService.DEBUG, "CAnouncements.renderXML(): Defering to servant render");
            }
            else {
                LogService.log(LogService.DEBUG, "CAnouncements.renderXML(): rendering announcements");
                Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());

                Document viewDoc = (Document) session.runtimeData.get(XML_DOC_KEY);
                       //ViewXML.get(runtimeData, staticData);
                long time2 = Calendar.getInstance().getTime().getTime();
                XSLT xslt = new XSLT(this);
                xslt.setXML(viewDoc);
                xslt.setTarget(out);
                xslt.setStylesheetParameter("baseActionURL", session.runtimeData.getBaseActionURL());
                char c1 = '-';
                char c2 = ':';
                char c3 = ' ';
                if (now != null) {
                    char[] exchars = now.toString().toCharArray();
                    StringBuffer exb = new StringBuffer();
                    for (int e = 0; e < exchars.length; e++) {
                        if ((exchars[e] != c1) && (exchars[e] != c2) && (exchars[e]
                                != c3)) {
                            exb.append(exchars[e]);
                        }
                    }
                    xslt.setStylesheetParameter("currentTime", exb.toString());
                }
                if (session.runtimeData.getParameter("topicID") != null) {
                    xslt.setStylesheetParameter("topicID", session.runtimeData.getParameter("topicID"));
                }
                if (session.fromViews.get(session.runtimeData.getParameter("view"))
                        != null) {
                    xslt.setStylesheetParameter("fromView", (String)session.fromViews.get(session.runtimeData.getParameter("view")));
                }
                else {
                    xslt.setStylesheetParameter("fromView", defaultView);
                }
                if (session.runtimeData.getParameter("messageID") != null) {
                    xslt.setStylesheetParameter("messageID", session.runtimeData.getParameter("messageID"));
                }
                if (session.runtimeData.getParameter("commandResponse") != null) {
                    xslt.setStylesheetParameter("commandResponse", session.runtimeData.getParameter("commandResponse"));
                }
                if (session.staticData.getPerson().getSecurityContext().isAuthenticated()){
                   xslt.setStylesheetParameter("isAuthenticated", "true");
                }
                Properties prps = PropertiesLoader.getProperties(OWNER, PROPERTIES);
                if (session.isAdminUser || (prps != null && prps.getProperty("usePermissions").equals("false"))) {
                    xslt.setStylesheetParameter("ignorePermissions", "true");
                }
                session.updateSender = prps.getProperty("updateSender");

                  // Set page number, defaults to 1 if no page number was sent
                String pageNumber = session.runtimeData.getParameter("pageNumber");
                pageNumber = (pageNumber == null) ? "1": pageNumber;
                xslt.setStylesheetParameter("pageNumber", pageNumber);

                if (session.isTopicAdminUser || session.isAdminUser) {
                    xslt.setStylesheetParameter("isTopicAdmin", "true");

                                            }


                xslt.setXSL(sslLocation, session.runtimeData.getParameter("view"),
                        session.runtimeData.getBrowserInfo());
                try {
                    xslt.transform();
                } catch (Exception e) {
                    LogService.log(LogService.ERROR, e);
                }
                long time3 = Calendar.getInstance().getTime().getTime();
                LogService.log(LogService.DEBUG, "CAnnouncements.renderXML() timer: "
                        + String.valueOf((time3 - time1)) + " ms total, xsl took "
                        + String.valueOf((time3 - time2)) + " ms for view " + session.runtimeData.getParameter("view"));
                LogService.log(LogService.DEBUG, "CAnnouncements.renderXML() time since setRD: "
                        + String.valueOf((time3 - session.startRD)));
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
          finally{
                           // clear out runtime references

                           session.runtimeData.remove(XML_DOC_KEY);
                           session.runtimeData = null;
                   }
        return;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getOwnerName () {
        return  "Announcements Channel (Columbia)";
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String[] getActivityTokens () {
        return  (String[])activities.keySet().toArray(new String[0]);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getOwnerToken () {
        return  OWNER;
    }

    /**
     * put your documentation comment here
     * @param token
     * @return
     */
    public String getActivityName (String token) {
        return  (String)activities.get(token);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String[] getTargetTokens () {
        return  TopicManager.instance().getAllTopicKeys();
    }

    /**
     * put your documentation comment here
     * @param token
     * @return
     */
    public String getTargetName (String token) {
        return  TopicManager.instance().getTopicName(token);
    }

    /**
     * put your documentation comment here
     * @param uid
     * @return
     */
    public ChannelCacheKey generateKey (String uid) {
        CAnnouncementsSessionData session = getSessionData(uid);
        ChannelCacheKey cck;
        if (session.servantChannel == null) {
            cck = new ChannelCacheKey();
            cck.setKey(uid);
          //  cck.setKeyValidity(getSessionData(uid).myvalidity);
           cck.setKeyValidity(session.myvalidity);
            LogService.log(LogService.DEBUG, "CAnnouncements.generateKey() : set validity to "
                    + String.valueOf(session.myvalidity));
        }
        else {
            cck = ((ICacheable)session.servantChannel).generateKey();
        }
        return  cck;
    }

    /**
     * put your documentation comment here
     * @param validity
     * @param uid
     * @return
     */
    public boolean isCacheValid (Object validity, String uid) {
        CAnnouncementsSessionData session = getSessionData(uid);
        boolean valid = false;
        if (session.servantChannel == null) {
            if (validity != null) {
                if (!((Timestamp)validity).before(session.myvalidity)) {
                    valid = true;
                }
            }
            long time3 = Calendar.getInstance().getTime().getTime();
            LogService.log(LogService.DEBUG, "CAnnouncements.isCacheValid() time since setRD: "
                    + String.valueOf((time3 - session.startRD)) + ", valid=" + valid);
        }
        else {
            valid = ((ICacheable)session.servantChannel).isCacheValid(validity);
        }

          if(valid){
                           // renderXML will not be called, so let's clear runtime references

                             session.runtimeData.remove(XML_DOC_KEY);
                           session.runtimeData = null;
                   }
        return  valid;
    }

    /**
     * put your documentation comment here
     * @return
     */
    private IAuthorizationPrincipal[] hackPrincipals () {
        //temporary hack until Groups Manager is ready to provide runtime principal selection
        IAuthorizationPrincipal[] iapa = new IAuthorizationPrincipal[0];
        try {
            ArrayList princs = new ArrayList();
            AuthorizationService as = AuthorizationService.instance();
            IAuthorizationPrincipal ap = as.newPrincipal("1", Class.forName("org.jasig.portal.security.IPerson"));
            princs.add(ap);
            IAuthorizationPrincipal ap2 = as.newPrincipal("2", Class.forName("org.jasig.portal.security.IPerson"));
            princs.add(ap2);
            IAuthorizationPrincipal ap3 = as.newPrincipal("0", Class.forName("org.jasig.portal.groups.IEntityGroup"));
            princs.add(ap3);
            IAuthorizationPrincipal ap4 = as.newPrincipal("4", Class.forName("org.jasig.portal.groups.IEntityGroup"));
            princs.add(ap4);
            iapa = (IAuthorizationPrincipal[])princs.toArray(iapa);
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        }
        return  iapa;
    }
}