/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 *
 */


package  edu.columbia.ais.portal.channels.announcements;

import  org.jasig.portal.*;
import java.lang.ref.SoftReference;
import  java.sql.Timestamp;
import  java.util.HashMap;
import  java.util.Iterator;
import  org.w3c.dom.Document;
import org.jasig.portal.groups.IGroupMember;


/**
 * A simple class acting as a holder for all data related to a single
 * user session
 *
 * @author Alex Vigdor
 * @version $Revision: 1.1 $
 */
public class CAnnouncementsSessionData {
    public String instanceIdentifier;
    public ChannelRuntimeData runtimeData;
    public ChannelStaticData staticData;
    public IGroupMember member;
    public IServant servantChannel;
    public Timestamp myvalidity;
    public long startRD;
    public HashMap fromViews = new HashMap(10);
//    public Document viewXML;
//   public Timestamp modViewXML;
    private SoftReference viewXML;
    public String uid;
    public boolean isAdminUser = false;
    //added
    public boolean isTopicAdminUser = false;
    public String updateSender = "T";
     private HashMap modViewXMLs = new HashMap();

               /**
                * get the XML model used to construct views for this session
                * @returns Document of session XML model.
                */
               public Document getViewXML(){
                   return (viewXML !=null) ? (Document) viewXML.get() : null;
}

     /**
                * set the XML model used to construct views for this session
                * @param Document the session XML model.
                */
                   public void setViewXML(Document doc){
                           viewXML = new SoftReference(doc);
                   }

               /**
                * Sets timestamp indicating last time XML was modified for specified topic.
                * @param topicID String value of topicID.
                * @param ts Timestamp of last XML change for topicID.
                */
               protected void setModViewXML (String topicID, Timestamp ts) {
                 modViewXMLs.put(topicID,  ts);
               }

               /**
                * Gets timestamp indicating last time XML was modified for specified topic.
                * @param topicID String value of topicID.
                * @returns Timestamp of last XML change for topicID.
                */
               protected Timestamp getModViewXML (String topicID) {
                 return (Timestamp)modViewXMLs.get(topicID);
               }

               /**
                * Removes timestamp entry for parameter topicID.
                * @param topicID String value of topicID.
                */
               protected void removeModViewXML (String topicID) {
                 modViewXMLs.remove(topicID);
               }

               /**
                * Gets maximum timestamp of last time XML was modified (latest change) across all topics.
                * @returns Timestamp of last XML change for topicID.
                */
               protected Timestamp getMaxModViewXML () {
                 Timestamp ts, tsMax = null;
                 Iterator it = modViewXMLs.values().iterator();
                 while (it.hasNext()) {
                   ts = (Timestamp)it.next();
                   if (tsMax == null || ts.after(tsMax)) {
                     tsMax = ts;
                   }
                 }
                 return tsMax;
               }

           }