/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;

import org.jasig.portal.services.*;
// import org.w3c.dom.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import javax.xml.parsers.*;


/**
 *
 * An ITopicStoreReadOnly imlementation to allow the publication of
 * topics retrieved via XML over HTTP from a remote portal using
 * the Columbia Announcements system
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.1 $
 */
public class HTTPTopicStore
        implements ITopicStoreReadOnly, Constants {

    private final String description = HTTP_TOPIC_STORE_DESC;
    protected String key = null;

    /** Creates new TopicStoreHTTP */
    public HTTPTopicStore () {
    }

    /**
     * put your documentation comment here
     * @param sb
     * @return
     */
    protected String filterXML (String sb) {
        try {
            int iBegin = 0;
            int iEnd = sb.length();
            ;
            if (sb.indexOf("<?xml") >= 0) {
                iBegin = sb.indexOf("<?xml");
            }
            return  sb.substring(iBegin, iEnd);
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
        return  null;
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void retrieveAllAnnouncements (Topic t) {
        try {
            String sLine = "";
            String xml = null;
            Document topicXML = null;
            try {
                DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                dbf.setValidating(false);
                dbf.setNamespaceAware(false);
                dbf.setIgnoringElementContentWhitespace(true);
                topicXML = dbf.newDocumentBuilder().parse(t.getArgument());
            } catch (Exception e) {
                LogService.log(LogService.ERROR, "TopicStoreHTTP.retrieveAllAnnouncements(): unable to retrieve topic xml from source "
                        + t.getArgument() + "\nroot cause: " + e);
            }

            if (topicXML != null) {
                Element root = topicXML.getDocumentElement();
                NodeList titles = root.getElementsByTagName("dc:title");
                if ((titles.getLength() > 0) && (t.getTitle().trim().length() < 1)) {
                    t.setTitle(titles.item(0).getFirstChild().getNodeValue());
                }
                NodeList descs = root.getElementsByTagName("dc:description");
                if ((descs.getLength() > 0) && (t.getDescription().trim().length() < 1)) {
                    t.setDescription(descs.item(0).getFirstChild().getNodeValue());
                }
                NodeList langs = root.getElementsByTagName("dc:language");
                if ((langs.getLength() > 0) && (t.getLanguage().trim().length() < 1)) {
                    t.setLanguage(langs.item(0).getFirstChild().getNodeValue());
                }
                NodeList creators = root.getElementsByTagName("dc:creator");
                if ((creators.getLength() > 0) && (t.getCreator().trim().length() < 1)) {
                    t.setCreator(creators.item(0).getFirstChild().getNodeValue());
                }
                NodeList pubs = root.getElementsByTagName("dc:publisher");
                if ((pubs.getLength() > 0) && (t.getPublisher().trim().length() < 1)) {
                    t.setPublisher(pubs.item(0).getFirstChild().getNodeValue());
                }
                NodeList refreshes = root.getElementsByTagName("refresh");
                if ((refreshes.getLength() > 0) && (t.getRefreshPeriod().trim().length() < 1)) {
                    t.setRefreshFrequency(Integer.parseInt(((Element)refreshes.item(0)).getAttribute("frequency")));
                    t.setRefreshPeriod(((Element)refreshes.item(0)).getAttribute("period"));
                }
                NodeList nl = root.getElementsByTagName("message");
                synchronized (t) {
                    t.setRetrievedAnnouncements(null);
                    t.setAnnouncements(new HashMap());
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element m = (Element)nl.item(i);
                        try {
                            t.addAnnouncement(new Announcement(t.getID(), m));
                        } catch (Exception e) {
                            LogService.log(LogService.ERROR, "TopicStoreHTTP.retrieveAllAnnouncements(): error adding announcement to topic "
                                    + e);
                        }
                    }
                    t.setRetrievedAnnouncements(new Timestamp(Calendar.getInstance().getTime().getTime()));
                }
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, "Announcements::TopicStoreHTTP:: Unable to retrieve remote XML for topic "
                    + t.getID() + "\n root cause: ");
            LogService.log(LogService.ERROR, e);
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Iterator retrieveAllTopics () {
        HashMap h = new HashMap();
        return  h.entrySet().iterator();
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getDescription () {
        return  this.description;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public boolean takesArg () {
        return  true;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getKey () {
        return  this.key;
    }

    /**
     * put your documentation comment here
     * @param key
     */
    public void setKey (String key) {
        this.key = key;
    }
}

