/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

// import  edu.columbia.ais.portal.channels.announcements.*;
import  edu.columbia.ais.portal.utils.*;
import  org.jasig.portal.groups.*;
import  java.util.HashMap;
import  java.util.Calendar;
import  java.util.Iterator;
import  java.util.ArrayList;
import  org.jasig.portal.RDBMServices;
import  org.jasig.portal.services.*;
import  org.jasig.portal.security.*;
import  java.sql.Connection;
import  java.sql.Statement;
import  java.sql.PreparedStatement;
import  java.sql.ResultSet;
import  java.sql.Timestamp;
import java.sql.Types;
import  java.io.StringReader;
import  java.io.StringWriter;
import  java.util.Calendar;
import  java.util.Hashtable;
import java.util.Set;
import  java.util.Vector;
import  java.util.Enumeration;
import  java.util.Iterator;
import  java.util.Map.Entry;
import java.util.Properties;

/**
 * ITopicStoreAuthority implementation - primary topic store for the
 * announcements system
 *
 * @author Alex Vigdor
 * @version $Revision: 1.1 $
 * modified by s. qiang, uhp (Esup-Portail) 2004/03/28
 * Last modified by s. qiang 2004/11/05
 */

public class RDBMTopicStore
        implements ITopicStoreAuthority, Constants {
    protected final String description = RDBM_TOPIC_STORE_DESC;
    //protected Hashtable topics = new Hashtable();
    protected String key = null;
    protected static Timestamp lastTopicModDate = null;
    protected static HashMap announcementCache = new HashMap();
    protected static HashMap announcementModDates = new HashMap();

    /** Creates new TopicStoreRDBMS */
    public RDBMTopicStore () {
    }

    /**
     * put your documentation comment here
     * @param a
     */
    public void storeAnnouncement (Announcement a) {
        //LogService.instance().log(LogService.DEBUG,"Saving Announcement");
        Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
        String sqlDraft = "F";
        if (a.getDraft()) {
            sqlDraft = "T";
        }

        Connection conn = null;
        RDBMServices.PreparedStatement ps = null;

        try {
            conn = this.getConnection();
            if (a.getID() == null) {
              a.setID(SequenceGenerator.instance().getNext(OWNER + ".Announcement"));
              StringBuffer sb = new StringBuffer("insert into " + MESSAGE_TABLE);
              sb.append(" (MESSAGE_ID,TOPIC_ID , SENDER, PRIORITY, TITLE, MESSAGE_SUMMARY, MESSAGE_BODY,");
              sb.append(" LINK_URI, IS_DRAFT,DATE_CREATED,DATE_APPEARS, ");
              sb.append(" DATE_EXPIRES, LAST_MOD_DATE) ");
              sb.append(" VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)");

              ps = new RDBMServices.PreparedStatement(conn, sb.toString());
              ps.clearParameters();
              ps.setString(1, a.getID());
              ps.setString(2, a.getTopicID());
              ps.setString(3, a.getFrom());
              ps.setInt(4, a.getPriority());
              ps.setString(5, a.getTitle());
              ps.setString(6, a.getSummary());
              ps.setString(7, a.getContent());
              ps.setString(8, a.getLink());
              ps.setString(9, sqlDraft);
              ps.setTimestamp(10, a.getCreated());
              ps.setTimestamp(11,a.getAppears());
              ps.setTimestamp(12, a.getExpires());
              ps.setTimestamp(13, now);

            }
            else {
                // old announcement - perform update
                    LogService.log(LogService.DEBUG, "old announcement, update it");
                StringBuffer sb = new StringBuffer("update " + MESSAGE_TABLE + " ");
                sb.append("set TOPIC_ID = ?, SENDER=?, PRIORITY=?, TITLE = ?, MESSAGE_SUMMARY = ?, MESSAGE_BODY = ?,");
                sb.append("LINK_URI= ?, IS_DRAFT=?, DATE_APPEARS=?, ");
                sb.append("DATE_EXPIRES = ?, LAST_MOD_DATE=? where MESSAGE_ID=?");

                ps = new RDBMServices.PreparedStatement(conn, sb.toString());
                ps.clearParameters();
                ps.setString(1, a.getTopicID());
                ps.setString(2, a.getFrom());
                ps.setInt(3, a.getPriority());
                ps.setString(4, a.getTitle());
                ps.setString(5, a.getSummary());
                ps.setString(6, a.getContent());
                ps.setString(7, a.getLink());
                ps.setString(8, sqlDraft);
                ps.setTimestamp(9, a.getAppears());
                ps.setTimestamp(10, a.getExpires());
                ps.setTimestamp(11, now);
                ps.setString(12, a.getID());

            }
            LogService.log(LogService.DEBUG, "RDBMTopicStore.storeAnnouncement(): " + ps);
            int rc = ps.executeUpdate();
            ps.close();

            //stmt.close();
            HashMap anns = getCache(a.getTopicID());
            anns.put(a.getID(), a);
        } catch (Exception e) {
            LogService.log(LogService.ERROR,"RDBMTopicStore.storeAnnouncement(): " + ps);
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void storeTopic (Topic t) {
        Connection conn = null;

        try {
            conn = this.getConnection();
            RDBMServices.PreparedStatement ps = null;
            char c1 = '\'';
            char c2 = '`';
            Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());

            if (t.getID() == null) {
                t.setID(SequenceGenerator.instance().getNext(OWNER + ".Topic"));
                IEntityGroup subscribers = GroupService.newGroup(Class.forName(SUBSCRIBER_CLASS));
                subscribers.setCreatorID(OWNER);
                subscribers.setName(t.getTitle() + " souscripteurs");
                subscribers.setDescription("Souscripteurs du canal Annonces pour "
                        + t.getTitle());
                subscribers.update();

                Properties prps = PropertiesLoader.getProperties(OWNER, PROPERTIES);
                String gname = prps.getProperty("rootGroupSubscriber");
                IEntityGroup rootSubscribersGroup = null;
                if (gname != null && !gname.trim().equals("")) {
                        rootSubscribersGroup = GroupService.findGroup(gname);

                        if (rootSubscribersGroup != null) {
                                    rootSubscribersGroup.addMember((IGroupMember) subscribers);
                                                rootSubscribersGroup.updateMembers();
                                                rootSubscribersGroup.update();
                                                    }
                                }
                else {
                          LogService.log(LogService.ERROR, "Announcement :: Erreur : Pb de groupe root (non existant ?) pour les groupes souscripteurs");
                }
                StringBuffer sb = new StringBuffer("INSERT INTO ");
                sb.append(TOPIC_TABLE).append("( TOPIC_ID, TOPIC_STORE, PUBLISH_REMOTE, ");
                sb.append("SUBSCRIBER_GROUP, ARGUMENT, TITLE, DESCRIPTION, LANGUAGE, CREATOR, ");
                sb.append("SUBJECT, PUBLISHER, DATE_STAMP, REFRESH_PERIOD, REFRESH_FREQUENCY, LAST_MOD_DATE, CHANNEL_INSTANCE)");
                sb.append(" VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
                ps = new RDBMServices.PreparedStatement(conn, sb.toString());
                ps.clearParameters();
                ps.setString(1,t.getID());
                ps.setString(2,t.getTopicStore());
                if (t.getPublishRemote()) ps.setString(3,"T");
                  else ps.setString(3,"F");
                ps.setString(4,subscribers.getKey());
                ps.setString(5,t.getArgument());

                if (t.getTitle()!=null) ps.setString(6,t.getTitle());
                  else ps.setString(6,null);

                if (t.getDescription()!=null) ps.setString(7,t.getDescription());
                  else ps.setString(7,null);

                if (t.getLanguage()!=null) ps.setString(8,t.getLanguage());
                  else ps.setString(8,null);
                if (t.getCreator() != null) ps.setString(9,t.getCreator().replace(c1, c2));
                 else ps.setString(9,null);

                if (t.getSubject() != null) ps.setString(10,t.getSubject());
                  else ps.setString(10,null);
                if (t.getPublisher() != null) ps.setString(11,t.getPublisher().replace(c1, c2));
                  else ps.setString(11,null);
                ps.setTimestamp(12,now);
                ps.setString(13,t.getRefreshPeriod());
                ps.setInt(14,t.getRefreshFrequency());
                ps.setTimestamp(15,now);
                ps.setString(16,t.getInstance());
                LogService.log(LogService.DEBUG,sb.toString());
                ps.executeUpdate();
                ps.close();

                TopicManager.instance().refreshTopic(t, false);
                IUpdatingPermissionManager upm = AuthorizationService.instance().newUpdatingPermissionManager(OWNER);
                IAuthorizationPrincipal subprinc = AuthorizationService.instance().newPrincipal(subscribers.getKey(),
                        Class.forName("org.jasig.portal.groups.IEntityGroup"));
                IPermission subperm = upm.newPermission(subprinc);
                subperm.setActivity("Subscribe");
                subperm.setTarget(t.getID());
                subperm.setType("GRANT");
                IPermission[] perms = new IPermission[1];
                perms[0] = subperm;
                upm.addPermissions(perms);
            }
            else {
                StringBuffer sb = new StringBuffer("UPDATE ");
                sb.append(TOPIC_TABLE).append(" set PUBLISH_REMOTE=?, ARGUMENT=?");
                sb.append(", TITLE=?, DESCRIPTION=?, LANGUAGE=?");
                sb.append(", CREATOR=?,SUBJECT=?, PUBLISHER=?, REFRESH_PERIOD=?");
                sb.append(", REFRESH_FREQUENCY=?, LAST_MOD_DATE=? ");
                sb.append(" WHERE TOPIC_ID=?");

                ps = new RDBMServices.PreparedStatement(conn, sb.toString());
                ps.clearParameters();
                if (t.getPublishRemote()) ps.setString(1,"T");
                  else ps.setString(1,"F");
                ps.setString(2,t.getArgument());

                if (t.getTitle()!=null) ps.setString(3,t.getTitle());
                  else ps.setString(3,null);

                if (t.getDescription()!=null) ps.setString(4,t.getDescription());
                  else ps.setString(4,null);
                if (t.getLanguage()!=null) ps.setString(5,t.getLanguage().replace(c1, c2));
                  else ps.setString(5,null);
                if (t.getCreator() != null) ps.setString(6,t.getCreator().replace(c1, c2));
                  else ps.setString(6,null);

                if (t.getSubject() != null) ps.setString(7,t.getSubject());
                  else ps.setString(7,null);
                if (t.getPublisher() != null) ps.setString(8,t.getPublisher().replace(c1, c2));
                  else ps.setString(8,null);
                ps.setString(9,t.getRefreshPeriod());
                ps.setInt(10,t.getRefreshFrequency());
                ps.setTimestamp(11,now);
                ps.setString(12,t.getID());
                ps.executeUpdate();
                ps.close();
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
    }

    /**
     * put your documentation comment here
     * @param a
     */
    public void deleteAnnouncement (Announcement a) {
        Connection conn = null;
        try {
            conn = this.getConnection();
            Statement stmt = conn.createStatement();
            String q2 = "delete from " + MESSAGE_TABLE + " where MESSAGE_ID='" +
                    a.getID() + "'";
            stmt.executeUpdate(q2);
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void deleteTopic (Topic t) {
        Announcement[] anns = t.getAllAnnouncements();
        for (int i = 0; i < anns.length; i++) {
            this.deleteAnnouncement(anns[i]);
        }
        try {
            IEntityGroup subs = t.getSubscriberGroup();
            subs.delete();
            IUpdatingPermissionManager upm = AuthorizationService.instance().newUpdatingPermissionManager(OWNER);
            upm.removePermissions(upm.getPermissions(null, t.getID()));
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
        String q = "delete from " + TOPIC_TABLE + " where TOPIC_ID='" + t.getID()
                + "'";
        Connection conn = null;
        try {
            conn = this.getConnection();
            Statement stmt = conn.createStatement();
            stmt.executeUpdate(q);
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
    }

    protected synchronized HashMap getCache(String topicID){
        if (announcementCache.get(topicID) == null) {
            HashMap anns = new HashMap();
            announcementCache.put(topicID, anns);
        }
        return (HashMap)announcementCache.get(topicID);
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void retrieveAllAnnouncements (Topic t) {
      try {
            synchronized (t) {
                HashMap anns = getCache(t.getID());

                 // Delete from announcements cache and topic object announcements that were
                // deleted from database by another server.
                Set dbAnnKeys = getAnnIDs(t.getID());
                HashMap tmpMap = new HashMap();
                tmpMap.putAll(anns);
                Set annKeys = tmpMap.keySet();
                annKeys.removeAll(dbAnnKeys);
                if (!annKeys.isEmpty()) {
                  Iterator i = annKeys.iterator();
                  while (i.hasNext()) {
                    String annKey = (String)i.next();
                    anns.remove(annKey);
                    t.removeAnnouncement(annKey);
                    t.store();
                  }
                }

                Timestamp ts = (Timestamp)announcementModDates.get(t.getID());
                Connection conn = this.getConnection();
                String sQuery = "select * from " + MESSAGE_TABLE + " where TOPIC_ID=?";
                if (ts != null) {
                    sQuery = sQuery + " and LAST_MOD_DATE > ? ";
                }
                RDBMServices.PreparedStatement ps = new RDBMServices.PreparedStatement(conn, sQuery);
                ps.clearParameters();
                ps.setString(1, t.getID());
                if (ts != null) {
                  ps.setTimestamp(2, ts);
                }

                LogService.log(LogService.DEBUG, "RDBMTopicStore.retrieveAllAnnouncements(): " + ps);
                ResultSet rs = ps.executeQuery();

                while (rs.next()) {
                    Announcement a = new Announcement(t.getID());
                    a.setID(rs.getString("MESSAGE_ID"));
                    LogService.instance().log(LogService.DEBUG,"Annoucnements::TopicStoreRDBMS:: loaded announcement ID:"+a.getID().toString());
                    a.setFrom(rs.getString("SENDER"));
                    a.setTitle(rs.getString("TITLE"));
                    a.setSummary(rs.getString("MESSAGE_SUMMARY"));
                    a.setContent(rs.getString("MESSAGE_BODY"));
                    a.setPriority(rs.getInt("PRIORITY"));
                    if (rs.getString("LINK_URI") != null) {
                        a.setLink(rs.getString("LINK_URI"));
                    }
                    if (rs.getTimestamp("DATE_APPEARS") != null) {
                        a.setAppears(rs.getTimestamp("DATE_APPEARS"));
                    }
                    if (rs.getTimestamp("DATE_EXPIRES") != null) {
                        a.setExpires(rs.getTimestamp("DATE_EXPIRES"));
                    }
                    if (rs.getTimestamp("DATE_CREATED") != null) {
                        a.setCreated(rs.getTimestamp("DATE_CREATED"));
                    }
                    if (rs.getTimestamp("LAST_MOD_DATE") != null) {
                        a.setLastModDate(rs.getTimestamp("LAST_MOD_DATE"));
                    }
                    String draft = rs.getString("IS_DRAFT");
                    if (draft.equals("T")) {
                        a.setDraft(true);
                    }
                    else {
                        a.setDraft(false);
                    }
                    //HashMap anns = (HashMap) announcementCache.get(a.getTopicID());
                    anns.put(a.getID(), a);
                    //t.addAnnouncement(a);
                }
                rs.close();
                ps.close();
                t.setAnnouncements(anns);
                ts = new Timestamp(Calendar.getInstance().getTime().getTime());
                t.setRetrievedAnnouncements(ts);
                announcementModDates.put(t.getID(), ts);
                if (conn !=null) this.releaseConnection(conn);
            }

        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public synchronized Topic[] retrieveNewTopicDefs () {
        Topic[] topics = new Topic[0];
        Connection conn = null;
        try {
            TopicManager tm = TopicManager.instance();
            conn = this.getConnection();
            RDBMServices.PreparedStatement ps;

            long currentMillis = (Calendar.getInstance()).getTime().getTime();
            Timestamp testTime = new Timestamp(currentMillis - tm.getRefreshMillis());
            if (lastTopicModDate == null || testTime.after(this.lastTopicModDate)) {
                String q = "select * from " + TOPIC_TABLE;
                if (lastTopicModDate != null) {
                    q = q.concat(" where LAST_MOD_DATE > ?" );
                }
                q = q.concat(" order by LAST_MOD_DATE desc");
                ps = new RDBMServices.PreparedStatement(conn, q);
                ps.clearParameters();
                if (lastTopicModDate != null) {
                  ps.setTimestamp(1,lastTopicModDate);
                }
                //LogService.instance().log(q);
                ResultSet rs = ps.executeQuery();
                topics = topicsFromResultSet(rs);
                this.lastTopicModDate = new Timestamp(Calendar.getInstance().getTime().getTime());
                ps.close();
                //LogService.instance().log("TopicStoreRDBMS:: loaded latest topics and set lastTopicModDate to "+lastTopicModDate.toString());
            }
            else {
            //LogService.instance().log("TopicStoreRDBMS:: waiting for refresh period to expire before checking updated topics");
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
        return  topics;
    }

    /**
     * put your documentation comment here
     * @return
     */
    protected Connection getConnection () {
       return  RDBMServices.getConnection();
    }

    /**
     * put your documentation comment here
     * @param conn
     */
    protected void releaseConnection (Connection conn) {
        RDBMServices.releaseConnection(conn);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getDescription () {
        return  this.description;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getKey () {
        return  this.key;
    }

    /**
     * put your documentation comment here
     * @param key
     */
    public void setKey (String key) {
        this.key = key;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public boolean takesArg () {
        return  false;
    }

    /**
     * put your documentation comment here
     * @param t
     * @return
     */
    public Topic refreshTopic (Topic t) {
        Connection conn = null;
        try {
            synchronized (t) {
                if (t.getID() != null) {
                    conn = this.getConnection();
                    //Statement stmt = conn.createStatement();
                    String q = "select * from " + TOPIC_TABLE + " where TOPIC_ID=?";
                    if (t.getLastModDate() != null) {
                        q = q + " and LAST_MOD_DATE > ?" ;
                    }

                    RDBMServices.PreparedStatement ps = new RDBMServices.PreparedStatement(conn, q);
                    ps.clearParameters();
                    ps.setString(1,t.getID());
                    if (t.getLastModDate() != null) {
                        ps.setTimestamp(2,t.getLastModDate());
                    }

                    ResultSet rs = ps.executeQuery();
                    Topic[] topics = topicsFromResultSet(rs);
                    ps.close();
                    if (topics.length > 0) {
                        t = (Topic)topics[0];
                    }
                    else {
                    //LogService.instance().log("Annoucnements::TopicStoreRDBMS:: no need to refresh");
                    }
                }
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
        return  t;
    }

    /**
     * put your documentation comment here
     * @param rs
     * @return
     */
    protected Topic[] topicsFromResultSet (ResultSet rs) {
        int i = 0;
        ArrayList topics = new ArrayList();
        try {
            while (rs.next()) {
                //if(i==0){
                //    this.lastTopicModDate = rs.getTimestamp("lastmoddate");
                //}
                Topic t = new Topic();
                t.setID(rs.getString("TOPIC_ID"));
                t.setTopicStore(rs.getString("TOPIC_STORE"));
                if (rs.getString("PUBLISH_REMOTE").equals("T")) {
                    t.setPublishRemote(true);
                }
                else {
                    t.setPublishRemote(false);
                }
                t.setArgument(rs.getString("ARGUMENT"));
                t.setTitle(rs.getString("TITLE"));
                t.setDescription(rs.getString("DESCRIPTION"));
                t.setLanguage(rs.getString("LANGUAGE"));
                t.setCreator(rs.getString("CREATOR"));
                t.setSubject(rs.getString("SUBJECT"));
                t.setPublisher(rs.getString("PUBLISHER"));
                t.setInstance(rs.getString("CHANNEL_INSTANCE"));
                t.setSubscribersKey(rs.getString("SUBSCRIBER_GROUP"));
                t.setDate(rs.getTimestamp("DATE_STAMP"));
                if (rs.getInt("REFRESH_FREQUENCY") > 0) {
                    t.setRefreshFrequency(rs.getInt("REFRESH_FREQUENCY"));
                }
                t.setRefreshPeriod(rs.getString("REFRESH_PERIOD"));
                t.setLastModDate(rs.getTimestamp("LAST_MOD_DATE"));
                topics.add(t);
                //LogService.instance().log(LogService.DEBUG,"fetched topic #"+t.getID().toString()+" from DB");
                i++;
            }
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        }
        return  (Topic[])topics.toArray(new Topic[0]);
    }


     /**
     * Retrieve all topic IDs from database and return in Set.
     * @return Set containing topic IDs as keys.
     */

    public Set getTopicIDs() {
        HashMap topicIDs = new HashMap();
        Connection conn = null;
        try {
          conn = this.getConnection();
          String q = "select TOPIC_ID from " + TOPIC_TABLE;
          RDBMServices.PreparedStatement ps = new RDBMServices.PreparedStatement(conn, q);
          ResultSet rs = ps.executeQuery();
          try {
              while (rs.next()) {
                  topicIDs.put(rs.getString("TOPIC_ID"), null);
              }
          } catch (Exception e) {
              LogService.instance().log(LogService.ERROR, e);
          }
          ps.close();
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
        return  (topicIDs.keySet());
    }

    /**
     * Retrieve all announcement IDs from database and return in Set.
     * @return Set containing announcement IDs as keys.
     */
    public Set getAnnIDs(String topicID) {
        HashMap annIDs = new HashMap();
        Connection conn = null;
        try {
          conn = this.getConnection();
          String q = "select MESSAGE_ID from " + MESSAGE_TABLE + " where TOPIC_ID = ?";
          RDBMServices.PreparedStatement ps = new RDBMServices.PreparedStatement(conn, q);
          ps.setString(1,topicID);
          ResultSet rs = ps.executeQuery();
          try {
              while (rs.next()) {
                  annIDs.put(rs.getString("MESSAGE_ID"), null);
              }
          } catch (Exception e) {
              LogService.instance().log(LogService.ERROR, e);
          }
          ps.close();
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        } finally {
            this.releaseConnection(conn);
        }
        return  (annIDs.keySet());
  }

}