/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

// import  edu.columbia.ais.portal.channels.announcements.*;

import  edu.columbia.ais.portal.utils.*;
import  java.util.Date;
import  java.util.Calendar;
import  java.util.Hashtable;
import  java.util.HashMap;
import  java.util.Iterator;
import  java.util.Collection;
import  java.io.*;
import  java.sql.Connection;
import  java.sql.Statement;
import  java.sql.ResultSet;
import  java.sql.Timestamp;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.ChannelRuntimeProperties;
import  org.jasig.portal.StylesheetSet;
import  org.jasig.portal.ChannelStaticData;
import  org.jasig.portal.groups.*;
import  org.jasig.portal.services.*;
import  org.w3c.dom.Node;
import  org.w3c.dom.NodeList;
import  org.w3c.dom.Element;
import  org.w3c.dom.Text;
import  org.w3c.dom.Document;
import javax.xml.parsers.*;
import  org.jasig.portal.security.IPerson;

/**
 * A class representing an individual topic, its XML representation
 * and its collection of announcements
 *
 * @author Alex Vigdor
 * @version $Revision: 1.1 $
 */
public class Topic
        implements Constants {
    String ID;
    String instance;
    String title;
    String description;
    String language;
    String creator;
    String subject;
    String publisher;
    Timestamp date;
    String refreshperiod;
    int refreshfrequency;
    boolean publishRemote;
    HashMap announcements = new HashMap();
    String argument;
    String topicstore;
    Timestamp lastModDate;
    Timestamp retrievedAnnouncements = null;
    //IEntityGroup subscribers = null;
    String subscribersKey;
    protected static Document blank;
    
    static{
      blank = org.jasig.portal.utils.DocumentFactory.getNewDocument();
    }

    /**
     * put your documentation comment here
     */
    public Topic () {
    }

    public Document retrieveXml () {
      return retrieveXml(true); 
    }
    
    public Element retrieveXml (Document doc) {
      return retrieveXml(true,doc); 
    }
    
    /**
     * put your documentation comment here
     * @return
     */
    public Document retrieveXml (boolean showHidden) {
      Document rDoc = (Document) blank.cloneNode(false);
      rDoc.appendChild(retrieveXml(showHidden,rDoc));
      return  rDoc;
    }

    public Element retrieveXml(boolean showHidden, Document rDoc){
      Element root = (Element)rDoc.createElement("topic");
      // renders topic to XML
      root.setAttribute("id", ID);
      root.setAttribute("store", this.getTopicStore());
      if (publishRemote) {
          root.setAttribute("publish", "global");
      }
      else {
          root.setAttribute("publish", "local");
      }
      if (argument != null) {
          root.setAttribute("argument", argument);
      }
      if (this.canEdit()) {
          root.setAttribute("type", "readwrite");
      }
      else {
          root.setAttribute("type", "readonly");
      }
      root.setAttribute("subscriberGroup", this.subscribersKey);
      //root.setAttribute("topicStore",this.topicstore);
      
      Element rdf = (Element)rDoc.createElement("rdf:RDF");
      rdf.setAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
      rdf.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
      Element rdfDesc = (Element)rDoc.createElement("rdf:Description");
      Element dcTitle = (Element)rDoc.createElement("dc:title");
      dcTitle.appendChild(rDoc.createTextNode(this.title));
      rdfDesc.appendChild(dcTitle);
      Element dcDescription = (Element)rDoc.createElement("dc:description");
      dcDescription.appendChild(rDoc.createTextNode(this.description));
      rdfDesc.appendChild(dcDescription);
      Element dcLanguage = (Element)rDoc.createElement("dc:language");
      dcLanguage.appendChild(rDoc.createTextNode(this.language));
      rdfDesc.appendChild(dcLanguage);
      rdf.appendChild(rdfDesc);
      root.appendChild(rdf);
      if ((this.refreshperiod != null)) {
          if ((this.refreshperiod.trim().length() > 0) && (refreshfrequency > 0)) {
              Element refresh = rDoc.createElement("refresh");
              refresh.setAttribute("period", this.refreshperiod);
              refresh.setAttribute("frequency", String.valueOf(refreshfrequency));
              root.appendChild(refresh);
          }
      }
      Announcement[] msgs = getAllAnnouncements();
      Date now = new Date();
      for (int ii = 0; ii < msgs.length; ii++) {
          if (showHidden || (!msgs[ii].getDraft() && 
                  ((msgs[ii].getAppears() == null) || (now.after(msgs[ii].getAppears()))) && 
                  ((msgs[ii].getExpires() == null) || (now.before(msgs[ii].getExpires()))) ) ){
            Element me = msgs[ii].retrieveXML(rDoc);
            root.appendChild(me);
         }
      }
      return root;
    }
    /**
     * put your documentation comment here
     * @param a
     */
    public void addAnnouncement (Announcement a) {
        if (a != null) {
            this.announcements.put(a.ID, a);
            update();
        }
        else {
            LogService.log(LogService.ERROR, "Topic.addAnnouncement(): bad attempt made to add announcement");
        }
    }

    /**
     * put your documentation comment here
     * @param as
     */
    protected void setAnnouncements (HashMap as) {
        announcements = as;
    }

    /**
     * put your documentation comment here
     * @param annID
     */
    public void removeAnnouncement (String annID) {
        if (announcements.get(annID) != null && canEdit()) {
            this.announcements.remove(annID);
            update();
        }
    }

    /**
     * put your documentation comment here
     * @param annID
     * @return
     */
    public Announcement getAnnouncement (String annID) {
        return  (Announcement)this.announcements.get(annID);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public IEntityGroup getSubscriberGroup () {
        IEntityGroup subscribers = null;
        if (subscribers == null && subscribersKey != null) {
            try {
                subscribers = GroupService.findGroup(subscribersKey);
            } catch (Exception e) {
                LogService.log(LogService.ERROR, e);
            }
        }
        return  subscribers;
    }

    /**
     * put your documentation comment here
     * @param key
     */
    public void setSubscribersKey (String key) {
        subscribersKey = key;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getID () {
        return  this.ID;
    }

    /**
     * put your documentation comment here
     * @param i
     */
    public void setID (String i) {
        if (lastModDate == null) {
            this.ID = i;
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getArgument () {
        return  this.argument;
    }

    /**
     * put your documentation comment here
     * @param s
     */
    public void setArgument (String s) {
        if (lastModDate == null) {
            argument = s;
        }
        else if (!s.equals(argument)) {
            this.argument = s;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getTitle () {
        return  this.title;
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void setTitle (String t) {
        if (lastModDate == null) {
            title = t;
        }
        else if (!t.equals(title)) {
            this.title = t;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getDescription () {
        return  this.description;
    }

    /**
     * put your documentation comment here
     * @param d
     */
    public void setDescription (String d) {
        if (lastModDate == null) {
            description = d;
        }
        else if (!d.equals(description)) {
            this.description = d;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getLanguage () {
        return  this.language;
    }

    /**
     * put your documentation comment here
     * @param l
     */
    public void setLanguage (String l) {
        if (lastModDate == null) {
            language = l;
        }
        else if (!l.equals(language)) {
            this.language = l;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public boolean getPublishRemote () {
        return  this.publishRemote;
    }

    /**
     * put your documentation comment here
     * @param b
     */
    public void setPublishRemote (boolean b) {
        if (lastModDate == null) {
            publishRemote = b;
        }
        else if (b != publishRemote) {
            this.publishRemote = b;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getTopicStore () {
        return  this.topicstore;
    }

    /**
     * put your documentation comment here
     * @param s
     */
    public void setTopicStore (String s) {
        if (lastModDate == null) {
            this.topicstore = s;
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public boolean canEdit () {
        boolean rval = false;
        TopicManager tm = TopicManager.instance();
        try {
            rval = tm.storeImplements(this.topicstore, "ITopicStoreAuthority");
            if (rval == false) {
                rval = tm.storeImplements(this.topicstore, "ITopicStoreReadWrite");
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
        return  rval;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getLastModDate () {
        return  lastModDate;
    }

    /**
     * put your documentation comment here
     * @param ts
     */
    public void setLastModDate (Timestamp ts) {
        this.lastModDate = ts;
    }

    /**
     * put your documentation comment here
     */
    protected void update () {
        this.setLastModDate(new Timestamp(Calendar.getInstance().getTime().getTime()));
    }

    /**
     * put your documentation comment here
     */
    public void store () {
        TopicManager tm = TopicManager.instance();
        ITopicStoreAuthority tsa = tm.getTopicStoreAuthority();
        tsa.storeTopic(this);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getRetrievedAnnouncements () {
        return  retrievedAnnouncements;
    }

    /**
     * put your documentation comment here
     * @param ts
     */
    public void setRetrievedAnnouncements (Timestamp ts) {
        this.retrievedAnnouncements = ts;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Announcement[] getAllAnnouncements () {
        return  (Announcement[])this.announcements.values().toArray(new Announcement[announcements.size()]);
    }

    /**
     * put your documentation comment here
     * @param period
     */
    public void setRefreshPeriod (String period) {
        if (lastModDate == null) {
            refreshperiod = period;
        }
        else if (!period.equals(refreshperiod)) {
            this.refreshperiod = period;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getRefreshPeriod () {
        return  this.refreshperiod;
    }

    /**
     * put your documentation comment here
     * @param frequency
     */
    public void setRefreshFrequency (int frequency) {
        if (lastModDate == null) {
            refreshfrequency = frequency;
        }
        else if (frequency != refreshfrequency) {
            this.refreshfrequency = frequency;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public int getRefreshFrequency () {
        return  this.refreshfrequency;
    }

    /**
     * put your documentation comment here
     * @param c
     */
    public void setCreator (String c) {
        if (lastModDate == null) {
            creator = c;
        }
        else if (!c.equals(creator)) {
            this.creator = c;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getCreator () {
        return  this.creator;
    }

    /**
     * put your documentation comment here
     * @param p
     */
    public void setPublisher (String p) {
        if (lastModDate == null) {
            publisher = p;
        }
        else if (!p.equals(publisher)) {
            this.publisher = p;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getPublisher () {
        return  this.publisher;
    }

    /**
     * put your documentation comment here
     * @param s
     */
    public void setSubject (String s) {
        if (lastModDate == null) {
            subject = s;
        }
        else if (!s.equals(subject)) {
            this.subject = s;
            update();
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String getSubject () {
        return  this.subject;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Timestamp getDate () {
        return  date;
    }

    /**
     * put your documentation comment here
     * @param d
     */
    public void setDate (Timestamp d) {
        if (lastModDate == null) {
            date = d;
        }
    }

    public String getInstance() {
        return instance;
    }

    public void setInstance(String instance) {
        this.instance = instance;
    }
}



