/**
 * Copyright ? 2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

import  edu.columbia.ais.portal.channels.announcements.*;
import  edu.columbia.ais.portal.utils.*;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.security.*;
import  org.jasig.portal.security.provider.*;
import  org.jasig.portal.groups.*;
import  org.jasig.portal.*;
import  org.jasig.portal.services.*;
import  java.sql.Timestamp;
import  java.util.Calendar;
import  java.util.Vector;
import  java.util.HashMap;
import  java.util.Iterator;
import  java.util.Map.Entry;
import  java.util.Properties;
import  java.io.StringReader;
import  java.io.StringWriter;
import  org.w3c.dom.Node;
import  org.w3c.dom.NodeList;
import  org.w3c.dom.Element;
import  org.w3c.dom.Text;
import org.w3c.dom.Document;
import javax.xml.parsers.*;


/**
 * A class that handles the generation and maintenace of user view
 * XML documents throughout the session
 *
 * @author Alex Vigdor
 * @version $Revision: 1.3 $
 */
public class ViewXML
        implements Constants {
    protected static HashMap priorities = new HashMap(5);
    static {
        priorities.put("lowest", "1");
        priorities.put("low", "2");
        priorities.put("normal", "3");
        priorities.put("high", "4");
        priorities.put("highest", "5");
    }

    /**
     * put your documentation comment here
     */
    private ViewXML () {
    }

    /**
     * put your documentation comment here
     * @param session
     * @return
     */
    private static synchronized Document getCachedDoc (CAnnouncementsSessionData session) throws Exception{
            Document d = session.getViewXML();
        //String key = String.valueOf(sd.getPerson().getID());
        if (d == null) {
            //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: creating fresh DocumentImpl");
            d = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
            Element r = d.createElement("CAnnouncements");
            d.appendChild(r);
            TopicManager tm = TopicManager.instance();
            ITopicStoreReadOnly[] allTopicStores = tm.getAllTopicStores();
            for (int i = 0; i < allTopicStores.length; i++) {
                ITopicStoreReadOnly it = allTopicStores[i];
                Element tse = d.createElement("topicStore");
                tse.setAttribute("description", it.getDescription());
                tse.setAttribute("key", it.getKey());
                if (it.takesArg()) {
                    tse.setAttribute("takesarg", "true");
                }
                if (it.getClass().getName().equals(DEFAULT_TOPIC_STORE)) {
                    tse.setAttribute("default", "true");
                }
                r.appendChild(tse);
            }
            IAuthorizationPrincipal ap = session.staticData.getAuthorizationPrincipal();
            Element apRoot = null;
            if (ap != null) {
                apRoot = d.createElement("principal");
                apRoot.setAttribute("token", ap.getPrincipalString());
                apRoot.setAttribute("type", ap.getType().getName());
                String name = ap.getKey();
                try {
                    name = EntityNameFinderService.instance().getNameFinder(ap.getType()).getName(name);
                } catch (Exception e) {
                    LogService.instance().log(LogService.ERROR, e);
                }
                apRoot.setAttribute("name", name);
                r.appendChild(apRoot);
            }

            Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
            Topic[] allTopics = tm.getAllTopics();
            for (int xyz = 0; xyz < allTopics.length; xyz++) {
                try {
                    Topic t = allTopics[xyz];
                    session.setModViewXML(t.getID(), now);
                    if(((t.getInstance()==null) && (session.instanceIdentifier==null))
                        || ((t.getInstance()!=null) && t.getInstance().equals(session.instanceIdentifier))
                        ){
                        processTopicSubscription(t, session,d);
                    }
                } catch (Exception e) {}
            }
            session.setViewXML(d);
        }
        else {
        //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: using cached DocumentImpl");
        }
        return  d;
    }

    /**
     * put your documentation comment here
     * @param topicID
     */
    public static void flushTopicByID (String topicID) {
        Iterator it = CAnnouncements.sessions.entrySet().iterator();
        Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
        while (it.hasNext()) {
            Entry e = (Entry)it.next();
            String key = (String)e.getKey();
            CAnnouncementsSessionData session = (CAnnouncementsSessionData)e.getValue();
            Document doc = session.getViewXML();
            if(doc!=null){
                                synchronized (session) {
                                        Element viewRoot = doc.getDocumentElement();
                                        Element[] delt = DOMHelper.getElementsByTagnameAndID(doc,
                                                        "topic", topicID);
                                        for (int i = delt.length - 1; i >= 0; i--) {
                                                viewRoot.removeChild(delt[i]);
                                        }
                                }
                                session.removeModViewXML(topicID);
            }
        }
    }

    /**
     * put your documentation comment here
     * @param t
     * @param session
     */
    public static void processTopicSubscription (Topic t, CAnnouncementsSessionData session, Document doc) {
        if (doc!= null) {
            try {
                Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
                Element viewRoot = doc.getDocumentElement();
                IEntityGroup tgrp = t.getSubscriberGroup();
                if (tgrp != null && tgrp.contains(session.member)) {
                    Element[] ss = DOMHelper.getElementsByTagnameAndID(doc,
                            "subscription", t.getID());
                    if (!(ss.length > 0)) {
                        Element s = doc.createElement("subscription");
                        s.setAttribute("id", t.getID());
                        s.setAttribute("inherited", "false");
                        viewRoot.appendChild(s);
                        session.setModViewXML(t.getID(), now);
                        //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: created new subscription for" +t.getID());
                    }
                    else {
                    //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: using cached subscription for "+t.getID());
                    }
                }
                else if (tgrp != null && tgrp.deepContains(session.member)) {
                    Element[] ss = DOMHelper.getElementsByTagnameAndID(doc,
                            "subscription", t.getID());
                    if (!(ss.length > 0) || !(ss[0].getAttribute("inherited").equals("true"))) {
                        Element s = doc.createElement("subscription");
                        s.setAttribute("id", t.getID());
                        s.setAttribute("inherited", "true");
                        viewRoot.appendChild(s);
                        //
                        session.setModViewXML(t.getID(), now);
                        //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: created new inherited subscription for "+t.getID());
                    }
                    else {
                    //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: using cached inherited subscription for "+t.getID());
                    }
                }
                else {
                    Element[] ss = DOMHelper.getElementsByTagnameAndID(doc,
                            "subscription", t.getID());
                    if (ss.length > 0) {
                        viewRoot.removeChild(ss[0]);
                        session.setModViewXML(t.getID(), now);
                        //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: deleting subscription for "+t.getID());
                    }
                    else {
                    //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: not subscribed to "+t.getID());
                    }
                }
            } catch (Exception e) {
                LogService.instance().log(LogService.ERROR, "CAnnouncements ViewXML:: unable to process subscroption "
                        + e);
            }
        }
    }

    /**
     * put your documentation comment here
     * @param session
     * @return
     */
    public static Document get (CAnnouncementsSessionData session){
        Document viewDoc = null;
        try {
          viewDoc = getCachedDoc(session);
          String userID = String.valueOf(session.staticData.getPerson().getID());
            Properties props = PropertiesLoader.getProperties(OWNER, PROPERTIES);
            TopicManager tm = TopicManager.instance();
            IEntity me = new EntityImpl(userID, Class.forName(SUBSCRIBER_CLASS));
            Topic[] allTopics = tm.getAllTopics();
            //LogService.instance().log(LogService.DEBUG,"CAnnouncements.ViewXML:: found "+String.valueOf(allTopics.length)+" topics");
            Element viewRoot = viewDoc.getDocumentElement();
            final IAuthorizationPrincipal ap = session.staticData.getAuthorizationPrincipal();
            Element apRoot = null;
            if (ap != null) {
                NodeList prlist = viewRoot.getElementsByTagName("principal");
                if (prlist.getLength() == 1) {
                    apRoot = (Element)prlist.item(0);
                }
            }
            Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
            //Timestamp userMod = (Timestamp) modCache.get(uid);
            //LogService.instance().log(LogService.DEBUG,"CAnnouncements.ViewXML:: Iterating over "+String.valueOf(allTopics.length)+" topics");
            for (int xyz = 0; xyz < allTopics.length; xyz++) {
                try {
                    Topic t = allTopics[xyz];
                    if(((t.getInstance()==null) && (session.instanceIdentifier==null))
                        || ((t.getInstance()!=null) && t.getInstance().equals(session.instanceIdentifier))
                        ){
                            Timestamp tmod = t.getLastModDate();
                            Element[] topicTest = DOMHelper.getElementsByTagnameAndID(viewDoc,
                                    "topic", t.getID());
                            if (tmod == null ||
                                session.getModViewXML(t.getID()) == null ||
                                tmod.after(session.getModViewXML(t.getID())) ||
                                topicTest.length == 0) {
                                //modCache.put(uid,now);
                                session.setModViewXML(t.getID(), now);
                                //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: loading from topic "+t.getID());
                                //Document tempDoc = t.retrieveXml();
                                Element tempRoot = t.retrieveXml(true,viewDoc);
                                Element[] oldts = DOMHelper.getElementsByTagnameAndID(viewDoc,
                                        "topic", t.getID());
                                if ((oldts.length > 0)) {
                                    Element oldElement = oldts[0];
                                    //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: deleting old topic "+t.getID());
                                    viewRoot.removeChild(oldElement);
                                }
                                //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: appending topic "+t.getID());
                                viewRoot.appendChild(tempRoot);
                                if (apRoot != null) {
                                    //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: Processing topic permissions . . .");
                                    NodeList oldPerms = viewDoc.getElementsByTagName("permission");
                                    for (int op = oldPerms.getLength() - 1; op >= 0; op--) {
                                        Element tperm = (Element)oldPerms.item(op);
                                        if (tperm.getAttribute("target").equals(String.valueOf(t.getID()))) {
                                            //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: removing cached permission");
                                            apRoot.removeChild(tperm);
                                        }
                                    }
                                    try {
                                        IPermission[] perms = ap.getAllPermissions(OWNER,
                                                null, String.valueOf(t.getID()));
                                        for (int yy = 0; yy < perms.length; yy++) {
                                            //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: adding topic permission");
                                            Element prm = viewDoc.createElement("permission");
                                            prm.setAttribute("principal", perms[yy].getPrincipal());
                                            prm.setAttribute("activity", perms[yy].getActivity());
                                            prm.setAttribute("type", perms[yy].getType());
                                            prm.setAttribute("target", perms[yy].getTarget());
                                            apRoot.appendChild(prm);
                                        }
                                    } catch (Exception e) {
                                        LogService.instance().log(LogService.ERROR, "CAnnouncements:ViewXML.get(): Error recording permissions"
                                                + e);
                                    }
                                }
                            }
                            else {
                            //LogService.instance().log(LogService.DEBUG,"Announcements::ViewXML:: using cached topic "+t.getID());
                            }
                    }
                } catch (Exception e) {
                    LogService.instance().log(LogService.ERROR, e);
                }
            }
            /*
             StringWriter sw = new java.io.StringWriter();
             XMLSerializer serial = new XMLSerializer(sw, new OutputFormat(viewDoc,"UTF-8",true));
             serial.serialize(viewDoc);
             LogService.instance().log(LogService.DEBUG,"CAnnouncements:viewXMl.getXML() ready:\n"+sw.toString());
             */

        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        }
        return  viewDoc;
    }
}
