/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */
package  edu.columbia.ais.portal.channels.announcements.commands;

import  edu.columbia.ais.portal.channels.announcements.commands.TopicalCommand;
import  edu.columbia.ais.portal.channels.announcements.*;
import  java.io.*;
import  java.util.*;
import  org.jasig.portal.IPermissible;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.ChannelRuntimeProperties;
import  org.jasig.portal.StylesheetSet;
import  org.jasig.portal.ChannelStaticData;
import  org.jasig.portal.services.LogService;
import  org.jasig.portal.security.*;
import  org.jasig.portal.security.provider.*;
import  org.jasig.portal.services.*;
import  org.jasig.portal.security.IPerson;


/**
 *
 * A class to handle topic editing
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.1 $
 */
public class EditTopic extends TopicalCommand {

    /**
     * put your documentation comment here
     * @param session
     * @param topic
     */
    protected void iExecute (CAnnouncementsSessionData session, Topic topic) {
        LogService.log(LogService.DEBUG, "Editing topic . . .");
        //Enumeration parms = rd.getHttpRequest().getParameterNames();
        TopicManager tm = TopicManager.instance();
        boolean isNew = false;
        try {
            String title = session.runtimeData.getParameter("title");
            if((title == null)  || (title.trim().length()==0)){
                session.runtimeData.setParameter("commandResponse", TOPIC_TITLE_EMPTY);

            }
            else{
                String arg = session.runtimeData.getParameter("argument" + session.runtimeData.getParameter("topicStore"));
                boolean servletpublish = false;
                if ((session.runtimeData.getParameter("servlet_publish") != null) &&
                        (session.runtimeData.getParameter("servlet_publish").equals("true"))) {
                    servletpublish = true;
                }
                if (topic == null) {
                    LogService.log(LogService.DEBUG, "creating new topic");
                    topic = new Topic();
                    topic.setInstance(session.instanceIdentifier);
                    isNew = true;
                }
                topic.setPublishRemote(servletpublish);
                if (arg != null) {
                    topic.setArgument(arg);
                }
                topic.setTopicStore(session.runtimeData.getParameter("topicStore"));
                topic.setTitle(session.runtimeData.getParameter("title"));
                topic.setDescription(session.runtimeData.getParameter("description"));
                topic.setLanguage(session.runtimeData.getParameter("language"));
                topic.setRefreshPeriod(session.runtimeData.getParameter("refresh_period"));
                if (session.runtimeData.getParameter("refresh_frequency").trim().length() > 0) {
                    topic.setRefreshFrequency(Integer.parseInt(session.runtimeData.getParameter("refresh_frequency")));
                }
                topic.store();
                //tm.refreshTopic(topic,true);
                session.runtimeData.setParameter("commandResponse", TOPIC_EDITED);
                LogService.log(LogService.DEBUG, "announcements.commands.EditTopic.iExecute():  is this new?");
                if (isNew) {
                    LogService.log(LogService.DEBUG, "announcements.commands.EditTopic.iExecute():  sure is!");
                    String tid = topic.getID();
                    session.runtimeData.setParameter("topicID", tid);
                    ArrayList perms = new ArrayList();
                    LogService.log(LogService.DEBUG, "announcements.commands.EditTopic.iExecute():  trying to make permissions");
                    try {
                        IUpdatingPermissionManager upm = AuthorizationService.instance().newUpdatingPermissionManager(OWNER);
                        IAuthorizationPrincipal ap = session.staticData.getAuthorizationPrincipal();
                        String[] activities = ((IPermissible)Class.forName(OWNER).newInstance()).getActivityTokens();
                        for (int a = 0; a < activities.length; a++) {
                            IPermission prm = upm.newPermission(ap);
                            prm.setActivity(activities[a]);
                            prm.setTarget(tid);
                            prm.setType("GRANT");
                            perms.add(prm);
                        }
                        LogService.log(LogService.DEBUG, "announcements.commands.EditTopic.iExecute():  adding "
                                + String.valueOf(perms.size()) + " permissions");
                        upm.addPermissions((IPermission[])perms.toArray(new IPermission[perms.size()]));
                    } catch (Exception e) {
                        LogService.log(LogService.ERROR, "announcments.commands.EditTopic.iExecute(): Unable to create permissions for new topic - topic "
                                + tid + " is now orphaned" + e);
                    }
                }
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }
}


