/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */


package  edu.columbia.ais.portal.channels.announcements.commands;

import  edu.columbia.ais.portal.channels.announcements.commands.TopicalCommand;
import  edu.columbia.ais.portal.channels.announcements.*;
import  edu.columbia.ais.portal.utils.*;
import  java.util.Enumeration;
import  java.util.Vector;
import  java.util.Iterator;
import  java.util.Map.Entry;
import  java.util.HashMap;
import  java.util.ArrayList;
import  java.io.*;
import  org.jasig.portal.services.*;
import  org.jasig.portal.groups.*;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.ChannelRuntimeProperties;
import  org.jasig.portal.StylesheetSet;
import  org.jasig.portal.ChannelStaticData;
import  org.jasig.portal.security.IPerson;


/**
 *
 * A class to handle subscribe/unsubscribe requests
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.1 $
 */
public class Subscribe extends TopicalCommand {

    /**
     * put your documentation comment here
     * @param session
     * @param topic
     */
    protected void iExecute (CAnnouncementsSessionData session, Topic topic) {
        LogService.instance().log(LogService.DEBUG, "Processing subscription request");
        try {
            Enumeration parms = session.runtimeData.getParameterNames();
            ArrayList subKeys = new ArrayList();
            TopicManager tm = TopicManager.instance();
            IGroupMember me = session.member;
            //SubscriptionManager sm = SubscriptionManager.instance();
            while (parms.hasMoreElements()) {
                String tp = (String)parms.nextElement();
                if (tp.startsWith("subscribe")) {
                    String ti = tp.substring(9);
                    Topic too = tm.getTopic(ti);
                    if (too != null) {
                        subKeys.add(too.getSubscriberGroup().getKey());
                        LogService.log(LogService.DEBUG, "processing request to join " +
                                ti);
                    }
                }
            }
            Topic[] alltopics = tm.getAllTopics();
            //IEntityGroup[] allgroups = tm.getAllSubscriberGroups();
            for (int i = 0; i < alltopics.length; i++) {
              Topic tt = alltopics[i];
                
                // condition IF added by Esup : bug for  topic subscribing in case of multiple instances
              if ((tt.getInstance()!=null) && tt.getInstance().equals(session.instanceIdentifier))
               {
            	// end of added  
               
                IEntityGroup thisGroup = tt.getSubscriberGroup();               //allgroups[i];
                boolean subscribe = false;
                boolean update = false;
                for (int j = 0; j < subKeys.size(); j++) {
                    if (thisGroup.getKey().equals(subKeys.get(j))) {
                        subscribe = true;
                        break;
                    }
                }
                if (subscribe) {
                    if (!thisGroup.contains(me)) {
                        LogService.log(LogService.DEBUG, "Joining subscriber group " +
                                thisGroup.getKey());
                        try {
                            thisGroup.addMember(me);
                            update = true;
                        } catch (Exception e) {
                            LogService.log(LogService.ERROR, e);
                        }
                    }
                }
                else {
                    if (thisGroup.contains(me)) {
                        LogService.log(LogService.DEBUG, "quitting subscriber group " +
                                thisGroup.getKey());
                        try {
                            thisGroup.removeMember(me);
                            update = true;
                        } catch (Exception e) {
                            LogService.log(LogService.ERROR, e);
                        }
                    }
                }
           
                if (update) {
                    LogService.log(LogService.DEBUG, "updating subscriber group " + thisGroup.getKey());
                    try {     
                        thisGroup.update();                       
                        //thisGroup.updateMembers();
                    } catch (Exception e) {
                        LogService.log(LogService.ERROR, e);
                    }
                    ViewXML.processTopicSubscription(tt, session, session.getViewXML());
                }
           // end of condition IF (added by Esup : bug correction for  topic subscribing in case of multiple instances)
               }
           // end of added
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
        LogService.log(LogService.DEBUG,"Finished Processing subscription request");
        session.runtimeData.setParameter("commandResponse", SUCCESS);
    }
}


