/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements.commands;

import  edu.columbia.ais.portal.channels.announcements.*;
import  edu.columbia.ais.portal.utils.*;
import  java.io.*;
import  org.jasig.portal.services.LogService;
import  org.jasig.portal.ChannelRuntimeData;
import  org.jasig.portal.ChannelRuntimeProperties;
import  org.jasig.portal.StylesheetSet;
import  org.jasig.portal.ChannelStaticData;
import  org.jasig.portal.security.IPerson;


/**
 *
 * An abstract class providing shared functionality to Announcements commands
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.1 $
 */
abstract class TopicalCommand
        implements IAnnouncementCommand, Constants {

    /**
     * put your documentation comment here
     * @param session
     */
    public void execute (CAnnouncementsSessionData session) {
        //LogService.instance().log(LogService.DEBUG,"abstract command calling concrete");
        Topic topic = null;
        if (session.runtimeData.getParameter("topicID") != null) {
            TopicManager tm = TopicManager.instance();
            topic = tm.getTopic(session.runtimeData.getParameter("topicID"));
        }
        try {
            if (topic != null) {
                synchronized (topic) {
                    this.iExecute(session, topic);
                }
            }
            else {
                this.iExecute(session, topic);
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }

    /**
     * put your documentation comment here
     * @param session
     * @param topic
     */
    abstract protected void iExecute (CAnnouncementsSessionData session, Topic topic);
}


