/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */


package  edu.columbia.ais.portal.channels.announcements.commands;

import java.sql.Timestamp;
import java.util.Calendar;

import org.jasig.portal.services.EntityNameFinderService;
import org.jasig.portal.services.LogService;

import edu.columbia.ais.portal.channels.announcements.Announcement;
import edu.columbia.ais.portal.channels.announcements.CAnnouncementsSessionData;
import edu.columbia.ais.portal.channels.announcements.ITopicStoreReadOnly;
import edu.columbia.ais.portal.channels.announcements.ITopicStoreReadWrite;
import edu.columbia.ais.portal.channels.announcements.Topic;
import edu.columbia.ais.portal.channels.announcements.TopicManager;


/*
 *  inherits HttpServletRequest req from abstract class TopicalCommand
 *  expected parameters:
 *  "msg_body" - text of announcement
 *  "aMM"   - appears month
 *  "aDD"   - appears day
 *  "aYYYY" - appears year
 *  "eMM"   - expires month
 *  "eDD"   - expires day
 *  "eYYYY" - expires year
 *  "msg_link" - url of link to be viewed as attachment
 *  "draft" - true or null'
 *
 * @author Alex Vigdor
 *
 * @version $Revision: 1.2 $
 *
 * Modified by Esup 2005/08/15
 */

public class WriteAnnouncement extends TopicalCommand {

    /**
     * put your documentation comment here
     * @param session
     * @param topic
     */
    protected void iExecute (CAnnouncementsSessionData session, Topic topic) {

            LogService.log(LogService.DEBUG, "WriteAnnouncement : getting topic manager");

        TopicManager tm = TopicManager.instance();
        LogService.log(LogService.DEBUG, "WriteAnnouncement : getting topic store");
        ITopicStoreReadOnly tst = tm.getTopicStore(topic.getTopicStore());
        boolean isNewMsg = false;
        try {
            LogService.log(LogService.DEBUG, "checking implementation");
            if (tm.storeImplements(topic.getTopicStore(), "ITopicStoreReadWrite")) {
                LogService.log(LogService.DEBUG, "casting");
                ITopicStoreReadWrite ts = (ITopicStoreReadWrite)tst;
                //LogService.instance().log(LogService.DEBUG,"making null announcement");
                Announcement a = null;
                if (session.runtimeData.getParameter("messageID") != null) {
                    LogService.log(LogService.DEBUG, "Grabbing existing announcement object");
                    a = topic.getAnnouncement(session.runtimeData.getParameter("messageID"));

                }
                if (a == null) {
                    LogService.log(LogService.DEBUG, "Creating new announcement object in topic "
                            + topic.getID());
                    a = new Announcement(topic.getID());
                    isNewMsg = true;
                }
                try {
                    LogService.log(LogService.DEBUG, "WriteAnnouncement : Populating announcement object");
                    synchronized (a) {
                        Timestamp now = new Timestamp(Calendar.getInstance().getTime().getTime());
                        a.setTitle(session.runtimeData.getParameter("msg_title"));
                        a.setSummary(session.runtimeData.getParameter("msg_summary"));
                        a.setContent(session.runtimeData.getParameter("msg_body"));

                        if(isNewMsg || session.updateSender.equals("T")) {
                           a.setFrom(EntityNameFinderService.instance().getNameFinder(session.member.getType()).getName(session.member.getKey()));
                        }
                        a.setPriority(session.runtimeData.getParameter("priority"));
                        if (session.runtimeData.getParameter("draft") == null) {
                            a.setDraft(false);
                        }
                        else {
                            a.setDraft(true);
                            ;
                        }

                        String  tmp_y,  tmp_m, tmp_d;
                        if(session.runtimeData.getParameter("eYYYY").trim().equals("")) {
                             String  tmp;
                             int mInt, yInt;
                             if(session.runtimeData.getParameter("aYYYY").trim().equals("")) {
                                 tmp = Timestamp2NumberString (now);
                                 tmp_y = tmp.substring(0, 4);
                                 tmp_m = (String) tmp.substring(4, 6);
                                 tmp_d = tmp.substring(6, 8);
                                 mInt = Integer.valueOf(tmp_m).intValue();
                                 yInt = Integer.valueOf(tmp_y).intValue();
                                }
                              else {
                                 mInt = Integer.valueOf(session.runtimeData.getParameter("aMM")).intValue();
                                 yInt = Integer.valueOf(session.runtimeData.getParameter("aYYYY")).intValue();
                                 tmp_d = session.runtimeData.getParameter("aDD");
                                    }
                              yInt = (mInt==12 ? yInt+1 : yInt);
                              mInt = (mInt==12 ? 1 : mInt + 1);
                              tmp_y = new Integer(yInt).toString();
                              tmp_m = new Integer(mInt).toString();

                        }
                        else {
                                tmp_y = session.runtimeData.getParameter("eYYYY");
                                tmp_m = session.runtimeData.getParameter("eMM");
                                tmp_d = session.runtimeData.getParameter("eDD");

                        }
                        Timestamp exp = this.getTimestamp(tmp_y, tmp_m, tmp_d);
                        Timestamp app = this.getTimestamp(session.runtimeData.getParameter("aYYYY"),
                                session.runtimeData.getParameter("aMM"), session.runtimeData.getParameter("aDD"));
                        if (exp != null) {
                            a.setExpires(exp);
                        }
                        if (app != null) {
                            a.setAppears(app);
                        }
                        a.setCreated(now);

                        if (!session.runtimeData.getParameter("msg_link").equals("")
                                && !session.runtimeData.getParameter("msg_link").equals("http://")) {
                            a.setLink(session.runtimeData.getParameter("msg_link"));
                        }
                        else if (session.runtimeData.getParameter("msg_link").equals("")) {
                            a.setLink(session.runtimeData.getParameter("msg_link"));
                        }
                        a.setLastModDate(now);
                        ts.storeAnnouncement(a);
                        topic.addAnnouncement(a);
                        topic.store();
                    }
                } catch (Exception e) {
                    LogService.log(LogService.ERROR, e);
                }
                session.runtimeData.setParameter("commandResponse", SUCCESS);
            }
            else {
                session.runtimeData.setParameter("commandResponse", STORE_LACKS_WRITE_SUPPORT);
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }


    /**
     * put your documentation comment here
     * @param year
     * @param month
     * @param day
     * @return
     */
    private String Timestamp2NumberString (Timestamp ts) {
        String rs = "";
        char c1 = '-';
        char c2 = ':';
        char c3 = ' ';
        if (ts != null) {
            char[] exchars = ts.toString().toCharArray();
            StringBuffer exb = new StringBuffer();
            for (int e = 0; e < exchars.length; e++) {
                if ((exchars[e] != c1) && (exchars[e] != c2) && (exchars[e] != c3)) {
                    exb.append(exchars[e]);
                }
            }
            rs = exb.toString();
        }
        return  rs;
    }


    private Timestamp getTimestamp (String year, String month, String day) {
        Timestamp rTS = null;
        if (!(year.equals("")) && !(month.equals("")) && !(day.equals(""))) {
            LogService.log(LogService.DEBUG, "Creating date for " + month
                    + " / " + day + " / " + year);
            Calendar expDate = Calendar.getInstance();
            expDate.set(new Integer(year).intValue(), (new Integer(month).intValue()
                    - 1), new Integer(day).intValue(),0,0,1);
            rTS = new Timestamp(expDate.getTime().getTime());
        }

        return  rTS;
    }



}