<?xml version="1.0" encoding="UTF-8"?>
<!--
        TopicToRSS.xsl
        XSL Transform to generate RSS feeds from an CU Announcement channel's remote XML format.
        @author Keith Zantow kzantow@vt.edu
        Modification Esup 15/08/2005
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
        <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
        <xsl:param name="homeURL">--- Votre URL ---</xsl:param>
        <xsl:template match="/">
                <rss version="2.0">
                        <channel>
                                <title><xsl:value-of select="topic/RDF/Description/title" /></title>
                                <link><xsl:value-of select="$homeURL" /></link>
                                <description><xsl:value-of select="topic/RDF/Description/description" /></description>
                                <xsl:choose>
                                        <xsl:when test="topic/RDF/Description/language">
                                                <language><xsl:value-of select="topic/RDF/Description/language" /></language>
                                        </xsl:when>
                                        <xsl:otherwise>
                                                <language>en-us</language>
                                        </xsl:otherwise>
                                </xsl:choose>
                                <xsl:for-each select="topic/message">
                                        <xsl:sort select="substring(concat(property[@name = 'appears']/@value, @generated.on),
                                                                                0, string-length(@generated.on))" order="descending" />
                                        <item>
                                                <xsl:if test="document/title">
                                                        <title><xsl:value-of select="document/title" /> </title>
                                                </xsl:if>
                                                <xsl:choose>
                                                        <xsl:when test="document/data.reference/@href">
                                                                <link><xsl:value-of select="document/data.reference/@href" /></link>
                                                        </xsl:when>
                                                        <xsl:otherwise>
                                                                <link><xsl:value-of select="$homeURL" /></link>
                                                        </xsl:otherwise>
                                                </xsl:choose>
                                                <xsl:if test="document/summary">
                                                        <description><xsl:value-of select="document/summary" /></description>
                                                </xsl:if>
                                                <xsl:choose>
                                                        <xsl:when test="property[@name = 'appears']/@value">
                                                                <pubDate><xsl:call-template name="reformatAnnouncementsDate">
                                                                        <xsl:with-param name="date" select="property[@name = 'appears']/@value" />
                                                                </xsl:call-template></pubDate>
                                                        </xsl:when>
                                                        <xsl:when test="@generated.on">
                                                                <pubDate><xsl:call-template name="reformatAnnouncementsDate">
                                                                        <xsl:with-param name="date" select="@generated.on" />

                                                                </xsl:call-template></pubDate>
                                                        </xsl:when>
                                                        <xsl:when test="property[@name = 'aYYYY']">
                                                                <pubDate><xsl:value-of select="property[@name = 'aYYYY']/@value" />/<xsl:value-of select="property[@name = 'aMM']/@value" />/<xsl:value-of select="property[@name = 'aDD']/@value" /></pubDate>
                                                        </xsl:when>
                                                        <xsl:when test="property[@name = 'eYYYY']">
                                                                <pubDate><xsl:value-of select="property[@name = 'eYYYY']/@value" />/<xsl:value-of select="property[@name = 'eMM']/@value" />/<xsl:value-of select="property[@name = 'eDD']/@value" /></pubDate>
                                                        </xsl:when>
                                                </xsl:choose>
                                                <author> <xsl:value-of select="@from" />  </author>
                                        </item>
                                </xsl:for-each>
                                <!-- xsl:copy-of select="current()" / -->
                        </channel>
                </rss>
        </xsl:template>

        <xsl:template name="reformatAnnouncementsDate">
                <xsl:param name="date" />
                <xsl:value-of select="substring($date, 7, 2)" />/<xsl:value-of select="substring($date, 5, 2)" />/<xsl:value-of select="substring($date, 1, 4)" />
        </xsl:template>
</xsl:stylesheet>