<?xml version='1.0' encoding="utf-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:xmsg="urn:x-lexica:xmsg:message:1.0" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
    <!--<xsl:output method="html" indent="yes"/> -->
    <xsl:param name="baseActionURL"/>
    <xsl:param name="commandResponse"/>
    <xsl:param name="ignorePermissions" select="false()"/>
    <xsl:param name="currentTime"/>
    <xsl:param name="isAuthenticated" select="false()"/>
    <xsl:param name="fromView"/>
    <xsl:param name="topicID" />
    <xsl:param name="messageID" />
    <xsl:param name="pageNumber">1</xsl:param>
    <xsl:variable name = "subscribedTopics" select = "/CAnnouncements/topic[parent::*/subscription/@id=@id]" />
     <xsl:variable name="mediaPath" select="'media/edu/columbia/ais/portal/channels/announcements'" />
    <xsl:variable name = "visibleAnnouncements" select = "$subscribedTopics/message[((not (property[@name='appears']/@value)) or (number(property[@name='appears']/@value) &lt; (number($currentTime)))) and ((not (@expires)) or (number(@expires) &gt; (number($currentTime)))) and (not(property[@name='draft']/@value='true'))]" />


        <!-- Determine actual page number to view (to account for the possibility that the list of announcements may have changed ) -->
        <xsl:variable name = "page"><xsl:choose>
                <!-- If $topicID and $messageID provided determine the page number for the display -->
            <xsl:when test="boolean($subscribedTopics[@id = $topicID]) and boolean($visibleAnnouncements[@id = $messageID])">
                <xsl:choose>
                        <xsl:when test="CAnnouncements/preferences/sorting[@mode='title']">
                                <xsl:for-each select = "$visibleAnnouncements">
                                        <xsl:sort select="document/title/text()"/>
                                        <xsl:if test = "@id = $messageID"><xsl:value-of select="position()" /></xsl:if>
                                </xsl:for-each>
                        </xsl:when>
                        <xsl:otherwise>
                                <xsl:for-each select = "$visibleAnnouncements">
                                        <xsl:sort select="@generated.on" order="descending" data-type="number"/>
                                        <xsl:if test = "@id = $messageID"><xsl:value-of select="position()" /></xsl:if>
                                </xsl:for-each>
                        </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <!-- Else, if pageNumber is greater than the number of announcements display the last page -->
            <xsl:when test = "$pageNumber &gt; count($visibleAnnouncements)"><xsl:value-of select="count($visibleAnnouncements)" /></xsl:when>
            <!-- Else, use the pageNumber provided -->
            <xsl:otherwise><xsl:value-of select="$pageNumber" /></xsl:otherwise>
        </xsl:choose></xsl:variable>

    <xsl:variable name = "displayPerPage">1</xsl:variable><!-- Single Item Viewing -->
    <xsl:variable name = "firstItem" select = "number($displayPerPage * ($page - 1) + 1)" />
    <xsl:variable name = "lastItem" select = "number($displayPerPage + $firstItem - 1)" />

    <xsl:template match="/">
            <script type="text/javascript" language="Javascript">
            function writeDayOfWeek (timeStamp)
            {
                    var year = parseInt(timeStamp.substr(0,4));
                    var monthStr = timeStamp.substr(4,2);
                       if(monthStr.substr(0,1).match('0')){
                        monthStr=monthStr.substr(1,1);
                      }
                    var month = parseInt(monthStr);
                    var day = parseInt(timeStamp.substr(6,2));
                    var date = new Date();
                    date.setFullYear(year);
                    date.setMonth(month-1);
                    date.setDate(day);
                    var daysOfWeek =   ["Dimanche","Lundi","Mardi","Mercredi","Jeudi","Vendredi","Samedi"]
                    document.write(daysOfWeek[date.getDay()]);
            }
        </script>
                <xsl:if test="$commandResponse!=''">
                        <p style="text-align:center;margin:0 0 10px 0;padding:0px;">
                                <span class="uportal-channel-warning">
                                        <xsl:value-of select="$commandResponse"/>
                                </span>
                        </p>
                </xsl:if>

            <div style="margin:10px;width:100%;">
                 <table border="1" cellspacing="0" cellpadding="0" width="90%"><tr><td>
                <table border="0" cellpadding="2" cellspacing="2" width="100%"  style="padding:0px;margin: 20px 20px 0 0;">
                        <xsl:choose>
                                <xsl:when test="CAnnouncements/preferences/sorting[@mode='title']">
                                        <xsl:apply-templates select = "$visibleAnnouncements">
                                                <xsl:sort select="document/title/text()"/>
                                    </xsl:apply-templates>
                                </xsl:when>

                                <xsl:otherwise>
                                        <xsl:apply-templates select = "$visibleAnnouncements">
                                                <xsl:sort select="@generated.on" order="descending" data-type="number"/>
                                    </xsl:apply-templates>
                                </xsl:otherwise>
                        </xsl:choose>
                </table>

                        <xsl:variable name = "totalMessages" select = "count($visibleAnnouncements)" />
                        <br/>
                         <p align="center">
                        <table border="0" cellpadding="0" cellspacing="0" style="margin: 20px 20px 0px 0;width:95%;">
                                <tr>
                                        <td align="left" class="uportal-background-highlight"><span class="uportal-text-reversed"> <i>Visualisation <xsl:value-of select="$firstItem" />
                                                <xsl:if test = "not($firstItem = $totalMessages) and not($firstItem = $lastItem)">
                                                        <xsl:choose>
                                                                <xsl:when test="$totalMessages &gt; $lastItem">
                                                                         - <xsl:value-of select="$lastItem" />
                                                                </xsl:when>

                                                                <xsl:otherwise> - <xsl:value-of select="$totalMessages" /></xsl:otherwise>
                                                        </xsl:choose>
                                                </xsl:if>

                                                sur <xsl:value-of select="$totalMessages" /></i></span></td>
                                        <td style="text-align:right;" align="center" class="uportal-background-highlight">

                                                <xsl:variable name = "rawPages" select = "$totalMessages div $displayPerPage" />
                                                <xsl:variable name = "totalNumberOfPages">
                                                        <xsl:choose>
                                                                <xsl:when test="$rawPages &gt; round($rawPages)"><xsl:value-of select="round($rawPages) + 1" /></xsl:when>
                                                                <xsl:otherwise><xsl:value-of select="round($rawPages)" /></xsl:otherwise>
                                                        </xsl:choose>
                                                </xsl:variable>


                                                        <xsl:if test="$page &gt; 1">
                                                                <a href="{$baseActionURL}?view=Announcement&amp;pageNumber=1" title="consulter la première annonce"><img src="{$mediaPath}/debut.gif" border="0" ALT="première annonce"/></a> ||
                                                                <a href="{$baseActionURL}?view=Announcement&amp;pageNumber={number($page)-1}" title="consulter annonce précédente"><img src="{$mediaPath}/preced.gif"  border="0" alt="annonce précédente"/></a> ||
                                                        </xsl:if>



                                                <xsl:if test="$lastItem &lt; $totalMessages">
                                                        <a href="{$baseActionURL}?view=Announcement&amp;pageNumber={number($page)+1}" title="consulter annonce suivante"><img src="{$mediaPath}/suiv.gif"  border="0" alt="consulter annonce suivante"/></a> ||
                                                        <a href="{$baseActionURL}?view=Announcement&amp;pageNumber={$totalNumberOfPages}" title="consulter la dernière annonce"><img src="{$mediaPath}/fin.gif"  border="0" alt="consulter dernière annonce"/></a>
                                                        </xsl:if>



                                        </td>
                                </tr>
                                <tr>   <td colspan="2" style="text-align:center;">
                                 <p>
                                         <table width="95%" border="0" cellspacing="0" cellpadding="0"><tr><td height="2" width="5" class="uportal-background-dark"><xsl:text> </xsl:text></td></tr></table>
                         </p>

                                                <form action="{$baseActionURL}" method="post" style="margin:0px;padding:0px;">
                                                        <input type="hidden" name="view" value="{$fromView}"/>
                                                        <input type="hidden" name="topicID" value="{$topicID}"/>
                                                        <input type="submit" name="Return" value="Retour" title="Retour" class="uportal-button"></input>
                                                </form>
                                        </td>
                                </tr>
                        </table> </p>
                                            </td>
                                    </tr>
                            </table>
            </div>
    </xsl:template>

    <xsl:template match="message">
        <xsl:if test="(position() &gt;= $firstItem) and (position() &lt;= $lastItem)">
            <tr>
                    <td style="vertical-align:top;text-align:center;" width="20%">
                            <!--<img src="media/org/jasig/portal/channels/CGenericXSLT/bullet.gif" width="16" height="16"/> -->
                            <span title="Description: {../RDF/Description/description}">
                       <span class="uportal-channel-table-caption"> <IMG src="{$mediaPath}/carres.gif" />  <xsl:text> </xsl:text>   <xsl:value-of select="../RDF/Description/title" />   </span>
                            </span>   <hr width="95%" />
                    </td></tr>
                    <tr><td valign="top">
                           <span class="uportal-channel-strong"><IMG src="{$mediaPath}/btn.gif" border="0"/>   <xsl:text> </xsl:text>  <xsl:value-of select="document/title" /></span>
                                        <p style="margin:0 0 5px 0; padding:0 0 0 0;" class="uportal-text-small">
                                  <i>  Date de la création : <script>writeDayOfWeek("<xsl:value-of select="@generated.on" />")</script>,  </i>
                                    <xsl:call-template name = "formatTime" >
                                            <xsl:with-param name="time" select="@generated.on"></xsl:with-param>
                                    </xsl:call-template>    <i>  <xsl:text> Par : </xsl:text> <xsl:value-of select="@from" />  </i>
                                        </p>  <br/>

                            <span class="uportal-channel-subtitle">Résumé :</span>
                            <p style="margin:0 0 5px 0; padding:0 0 0 0;" class="uportal-channel-text">

                                    <xsl:variable name="text-to-break" select="document/summary/self::*" />
                                    <xsl:call-template name="break">
                                    <xsl:with-param name="txt" select="$text-to-break"/>
                                    </xsl:call-template>
                            </p>
                           <span class="uportal-channel-subtitle">Annonce :</span>
                            <p style="margin:0 0 5px 0; padding:0 0 0 0;" class="uportal-channel-text">
                                       <xsl:value-of select="document/data/self::*" disable-output-escaping="yes"/>
                            </p>
                            <xsl:apply-templates select="document/data.reference"/>
                    </td>
            </tr>
        </xsl:if>
    </xsl:template>

    <xsl:template match="document/data.reference">
      <xsl:if test="string-length(@href) &gt; 5">
        <p style="margin:20px 0 5px 0; padding:0 0 0 0;" class="uportal-channel-text">
               Consulter le lien :
                <a target="_blank" title="Ouvrir le lien dans une nouvelle fenetre">
                        <xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
                        <xsl:value-of select="@href"/>
                </a>
        </p>
      </xsl:if>
    </xsl:template>

      <xsl:template name="break">
   <xsl:param name="txt"/>
   <xsl:choose>
   <xsl:when test="contains($txt, '&#xa;')">
      <xsl:value-of select="substring-before($txt, '&#xa;')" disable-output-escaping="yes"/>
      <br/>
      <xsl:call-template name="break">
          <xsl:with-param name="txt" select="substring-after($txt, '&#xa;')"/>
      </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
        <xsl:value-of select="$txt" disable-output-escaping="yes"/>
   </xsl:otherwise>
   </xsl:choose>
</xsl:template>

    <xsl:template match="abstract">
            <xsl:copy-of select="./child::node()"/>
    </xsl:template>

    <xsl:template match="data">
            <xsl:copy-of select="./child::node()"/>
    </xsl:template>

        <xsl:template name="formatTime">
                <xsl:param name="time" />
                <!-- parse generated on to represent date-->
                <xsl:value-of select="substring($time,7,2)" />
                <xsl:text>/</xsl:text>
                <xsl:value-of select="substring($time,5,2)" />
                <xsl:text>/</xsl:text>
                <xsl:value-of select="substring($time,1,4)" />
                <xsl:text> </xsl:text>
                <xsl:value-of select="substring($time,9,2)" />
                <xsl:text>:</xsl:text>
                <xsl:value-of select="substring($time,11,2)" />
        </xsl:template>

</xsl:stylesheet>